/* $Header: /home/powell/mod10/TERRA/v5.0.1/RCS/MOD_PR10_AAmain.c,v 1.1 2005/05/19 17:35:31 powell Exp $

****************************************************************************************
!C

!Description:

MODIS Snow Algorithm Main program reads in TERRA MODIS L1B data, the
cloud mask data, and the sensor zenith angles in the GEO data.	It does a per-pixel
analysis for snow, then writes an output product.  The code uses the PGS toolkit, the
HDF SD API, and the HDF-EOS SW API.  Runs on a granule of L1B data.

Program returns:
	0 - Success (There will be no output file if there are no daytime scans.)
	1 - Error, no snow HDF output

!Input Parameters:
	in_file			MODIS L1B data granule
	SensorZenith_file	MODIS GEO corresponding to L1B granule
	CloudMask_file		MODIS CLOUD corresponding to the L1B granule

!Output Parameters:
   SDS Name: Snow_Cover
	long_name			"Snow covered land"
	units				"none"
	format				"I3"
	coordsys			"cartesian"
	valid_range			0,254
	_FillValue			255
	Nadir_data_resolution		"500 m"
	Valid EV Obs Band 1 (%)
	Valid EV Obs Band 2 (%)
	Valid EV Obs Band 4 (%)
	Valid EV Obs Band 6 (%)
	Saturated EV Obs Band 1 (%)
	Saturated EV Obs Band 2 (%)
	Saturated EV Obs Band 4 (%)
	Saturated EV Obs Band 6 (%)
	Key:

   SDS Name: Snow_Cover_Pixel_QA
	long_name			"Snow cover per pixel thematic QA"
	units				"none"
	format				"I3"
	coordsys			"cartesian"
	valid_range			0,254
	_FillValue			255
	Key:

   SDS Name: Fractional_Snow_Cover
        long_name			"Fractional snow cover, 500m"
        units				"none"
        format				"I3"
        coordsys			"cartesian"
	valid_range			0,254
	_FillValue			255
	Key:

   SDS Name: "Latitude"
	long_name			"Coarse 5 km resolution latitude"
	units				"degrees"
	valid_range			-90.0,90.0
	_FillValue			999.0
	source

   SDS Name: "Longitude"
	long_name			"Coarse 5 km resolution longitude"
	units				"degrees"
	valid_range			-180.0,180.0
	_FillValue			999.0
	source


!Revision History:

   $Log: MOD_PR10_AAmain.c,v $
   Revision 1.1  2005/05/19 17:35:31  powell
   Initial revision


!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

    Portions developed at the National Center for Supercomputing
    Applications at the Univ. of Illinois at Urbana-Champaign.

   Principal Investigator, Vincent Salomonson
        Vincent.V.Salomonson@NASA.gov
   Associate Investigator, Dorothy Hall
        Dorothy.K.Hall@NASA.gov
   Scientist/Programmer George Riggs
        George.A.Riggs.1@gsfc.nasa.gov
        301-614-5650
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706
   Programmer, Hugh W. Powell
        Hugh.W.Powell.1@gsfc.nasa.gov
        301-286-2310
        NASA/GSFC/Code 614.1
   Programmer, Nicolo E. DiGirolamo
        Nicolo.E.DiGirolamo.1.1@gsfc.nasa.gov
        301-614-5824
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706


!References and Credits:
        Major assistance with integrating PGS  toolkit, HDF,
	and HDF-EOS was provided by Gang Ye of SDST/STIG.

!Design Notes:
        This is the Collection 5 version  of the code.

!END
***************************************************************************************/

#include <stdio.h>
#include <string.h>
#include <time.h>

#include "PGS_MEM.h"
#include "PGS_MET.h"
#include "PGS_PC.h"
#include "hdf.h"
#include "mfhdf.h"
#include "HdfEosDef.h"

#define THIS_IS_MAIN_PROGRAM
#include "MOD_PR10.h"
#include "MOD_PR10_ActionMessages.h"

#define BAND1_DETECTOR_OFFSET	000  /* Offset for dead and noisy detectors in lists */
#define BAND2_DETECTOR_OFFSET	040  /* Offset for dead and noisy detectors in lists */
#define BAND4_DETECTOR_OFFSET	100  /* Offset for dead and noisy detectors in lists */
#define BAND6_DETECTOR_OFFSET	140  /* Offset for dead and noisy detectors in lists */
#define NUMBER_OF_DETECTORS	20

#define NORTH	(char)+1
#define SOUTH	(char)-1

/*** Prototypes ***/
void ProcessCloudFile( enum OpType );
void ProcessGeoFile  ( enum OpType );
void ProcessL1Bfile  ( enum OpType );
void ComputeSfcTemp  ( enum OpType );
void ProcessSnowFile ( enum OpType );
void Process5kmSnowFile ( enum OpType );
void ComputeSnow( void );
void ComputeFractionalSnow( void );
void CopyL1BmetaToSnow( void );
void CopyGEOmetaToSnow( void );
void MakeMeta( void );

/*** Global Variables ***/
char			RCSidentString[8192];
int32			NoSamps, No5kmSamps;	/* Column dimension of L1B earth view */
int32			No1kmSamps;		/* Column dimension of L1B earth view */
int32			NoScans, No5kmScans;	/* Row	  dimension of L1B earth view */
int32			No1kmScans;		/* Row	  dimension of L1B earth view */
int32			DayModeScans;		/* Exit program if no day mode scan   */
int32			ThisScanNo;		/* Present scan line being processed  */
char			HaltMsg[2048];
char			in_FileName[PGSd_PC_FILE_PATH_MAX];	/* L1B 500m file name  */
char			in1km_FileName[PGSd_PC_FILE_PATH_MAX];	/* L1B 1km file name  */
extern char		cloud_FileName[], geo_FileName[];
int8			NoisyDetectorFlag, DeadDetectorFlag;
struct tm		CurrentTime;
struct COUNTS		cnt, cntf;		/* For snow & fractional snow */
PGSt_MET_all_handles	Meta500Handles;
struct Meta5km		km;			/* Holder for 5 km meta data */
struct CHECK		ck;
extern int32		SnowHDFhandle;
extern int8		DeadDetectorList[], NoisyDetectorList[];

float64			aa[3], bb[3], cc[3], dd[3]; /* Used by sea ice by IST */

int32                   snow_rejected_warm=0, snow_rejected_scanangle=0 ; /* counters */

extern int		FracBig, FracNormal, FracLittle;
/*====================================================================================*/

int main()
{
  char *rcs="$Id: MOD_PR10_AAmain.c,v 1.1 2005/05/19 17:35:31 powell Exp $";

  PGSt_integer	version = 1;			     /* for HDF input file */
  time_t	TimeRtn;
  struct tm	*curtime;
  char		HoldValue[16];
  float32	HoldFloat32;

  /*** Get the RCS identification for the main program and include files. ***/
  sprintf( RCSidentString, "\t%s\n\t%s\n\t%s\n", rcs, rcs1, rcs2 );


  /*** Get current time for later use in making meta data. ***/
  if( (TimeRtn=time(NULL)) == -1 )
    HALT2(ACT08,"Bad time value returned from operating system");

  if( (curtime=(struct tm *)gmtime(&TimeRtn)) == 0 )
    HALT2(ACT08,"No time structure filled in call to 'gmtime'.");

  memcpy( &CurrentTime, curtime, sizeof(CurrentTime) );


  /*** Be sure MODIS is nadir pointing before running the program.  (No output file
  is made if it is not nadir pointing.) ***/
  {
    char	NadirStg[64] = {'\0'}, NadirValue[64] = {'\0'};
    void	*local = NadirStg, *local2 = NadirValue;

    version = 1;
    if(PGS_MET_GetPCAttr(L1B_INPUT, version, CORE, "AdditionalAttributeName.9",
			 &local ) != PGS_S_SUCCESS )
      HALT2(ACT01,"PGS_MET_GetPCAttr failed: AdditionalAttributeName.9");

    if( strcmp( NadirStg, "NadirPointing" ) != 0 )
      HALT2(ACT02,"AdditionalAttributeName.9 != 'NadirPointing'.");

    version = 1;
    if(PGS_MET_GetPCAttr(L1B_INPUT, version, CORE, "ParameterValue.9",
			 &local2 ) != PGS_S_SUCCESS )
      HALT2(ACT03,"PGS_MET_GetPCAttr failed: Parameter.9 (NadirPointing value)");

    if( strcmp( NadirValue, "Y" ) != 0 )
      HALT2(ACT07, "Instrument is not nadir pointing.  ABORTING the run.")
   }

  /*** Zero out the COUNTS structures. ***/
  memset(  &cnt.FirstEntryInStruct, 0, sizeof( struct COUNTS ) );
  memset( &cntf.FirstEntryInStruct, 0, sizeof( struct COUNTS ) );

  /*** Retrieve 500 meter input file name from PCF file  ***/
  version = 1;
  if(PGS_PC_GetReference(L1B_INPUT, &version, in_FileName) != PGS_S_SUCCESS)
    HALT2(ACT04,"Error getting L1B input file name from PCF");

  /*** Retrieve 1 km input file name from PCF file  ***/
  version = 1;
  if(PGS_PC_GetReference(L1B1KM_INPUT, &version, in1km_FileName) != PGS_S_SUCCESS)
    HALT2(ACT04,"Error getting L1B 1km input file name from PCF");

  /*** Retrieve geo file name from PCF file  ***/
  version = 1;
  if(PGS_PC_GetReference(GEO_INPUT,&version, geo_FileName) != PGS_S_SUCCESS)
    HALT2(ACT04,"Error getting geo input file name from PCF");

  /*** Retrieve cloud file name from PCF file	 ***/
  version = 1;
  if(PGS_PC_GetReference(CLOUD_INPUT,&version, cloud_FileName) != PGS_S_SUCCESS)
    HALT2(ACT04,"Error getting cloud input file name from PCF");



  /*** Open the L1B input file and do any necessary initialization. ***/
  ProcessL1Bfile( OPEN );
  ComputeSfcTemp( OPEN );

  if( DayModeScans == 0 ) {
    PGS_SMF_GenerateStatusReport("L1B has no day scans, no snow output file made." );
    return(0);
  }

  /*** Set the dimensions of the 5 km output file. ***/
  No5kmScans = NoScans/10;
  No5kmSamps = MAX_5KM_SAMPS;

  /*** Initialize the meta data output handles. ***/
  if(PGS_MET_Init(MCF_FILE, Meta500Handles) != PGS_S_SUCCESS)
    HALT2(ACT04,"Failure in PGS_MET_Init for 500 meter");


  /*** Create 500 meter output file here before doing any work. ***/
  ProcessSnowFile( OPEN );

  CopyL1BmetaToSnow();		/* Copy some PGE meta data from L1B input. */

  CopyGEOmetaToSnow();		/* Copy some PGE meta data from GEO input. */

  /*** Get the scan angles from the GEO file. ***/
  ProcessGeoFile( OPEN );

  /*** Read in the cloud mask SDS.  The first 3 bytes of the cloud mask are read. ***/
  ProcessCloudFile( OPEN );

  /*** Create 5 km output file here before doing any work. ***/
  Process5kmSnowFile( OPEN );


  /*** Be sure all three input granules have the same GRings. ***/
  {
    int		n;
    float64	lat_l1bgeo, lat_l1bcld, lon_l1bgeo, lon_l1bcld;
    float64	errP = 0.0001,   errM = -0.0001;




    for( n=0; n<MAX_GRING_POINTS; n++ ) {
      lat_l1bgeo = ck.l1b.GRingPointLatitude[n]  - ck.geo.GRingPointLatitude[n];
      lat_l1bcld = ck.l1b.GRingPointLatitude[n]  - ck.cld.GRingPointLatitude[n];
      lon_l1bgeo = ck.l1b.GRingPointLongitude[n] - ck.geo.GRingPointLongitude[n];
      lon_l1bcld = ck.l1b.GRingPointLongitude[n] - ck.cld.GRingPointLongitude[n];

      if( lat_l1bgeo > errP  || lat_l1bgeo < errM  ||
	  lat_l1bcld > errP  || lat_l1bcld < errM  ||
	  lon_l1bgeo > errP  || lon_l1bgeo < errM  ||
	  lon_l1bcld > errP  || lon_l1bcld < errM )
	HALT2(ACT04,"The GRingPoint latitude/longitudes of the 3 input files are not in agreement.");
    }
  }

  /*** Be sure all three input granules have the same RangeBeginning date/time. ***/
  if( strcmp( ck.l1b.RangeBeginningDate, ck.geo.RangeBeginningDate ) != 0  ||
      strcmp( ck.l1b.RangeBeginningDate, ck.cld.RangeBeginningDate ) != 0  ||
      strcmp( ck.l1b.RangeBeginningTime, ck.geo.RangeBeginningTime ) != 0  ||
      strcmp( ck.l1b.RangeBeginningTime, ck.cld.RangeBeginningTime ) != 0 )
    HALT2(ACT04,"The RangeBeginningDate/Time of the 3 input files are not in agreement");

  /*** Determine the hemisphere in which this granule is located.  Do this by
       comparing the northernmost and southernmost GRingPointLatitudes. ***/
  {
    float64	top, bot;
    char	hemi;

    /*** Northern Hemisphere ***/
    float64 n1[] = {-1.5711228087,1.0054774067,1.8532794923,-0.7905176303}; /* < 240   */
    float64 n2[] = {-2.3726968515,1.0086040702,1.6948238801,-0.2052523236}; /* 240-260 */
    float64 n3[] = {-4.2953046345,1.0150179031,1.9495254583, 0.1971325790}; /* > 260   */
    /*** Southern Hemisphere ***/
    float64 s1[] = {-0.1594802497,0.9999256454,1.3903881106,-0.4135749071}; /* < 240   */
    float64 s2[] = {-3.3294560023,1.0129459037,1.2145725772, 0.1310171301}; /* 240-260 */
    float64 s3[] = {-5.2073604160,1.0194285947,1.5102495616, 0.2603553496}; /* > 260   */

    top = bot = ck.l1b.GRingPointLatitude[0];
    top = top > ck.l1b.GRingPointLatitude[1] ? top : ck.l1b.GRingPointLatitude[1];
    top = top > ck.l1b.GRingPointLatitude[2] ? top : ck.l1b.GRingPointLatitude[2];
    top = top > ck.l1b.GRingPointLatitude[3] ? top : ck.l1b.GRingPointLatitude[3];
    bot = bot < ck.l1b.GRingPointLatitude[1] ? bot : ck.l1b.GRingPointLatitude[1];
    bot = bot < ck.l1b.GRingPointLatitude[2] ? bot : ck.l1b.GRingPointLatitude[2];
    bot = bot < ck.l1b.GRingPointLatitude[3] ? bot : ck.l1b.GRingPointLatitude[3];

    if( bot > 0 )
      hemi = NORTH;
    else if( top < 0 )
      hemi = SOUTH;
    else {
      if( top >= (-bot) )
	hemi = NORTH;
      else
	hemi = SOUTH;
    }

    /*** Set coefs for computing sea ice by IST. ***/
    if( hemi == NORTH ) {
      aa[0] = n1[0];	aa[1] = n2[0];	aa[2] = n3[0];
      bb[0] = n1[1];	bb[1] = n2[1];	bb[2] = n3[1];
      cc[0] = n1[2];	cc[1] = n2[2];	cc[2] = n3[2];
      dd[0] = n1[3];	dd[1] = n2[3];	dd[2] = n3[3];
    } else {
      aa[0] = s1[0];	aa[1] = s2[0];	aa[2] = s3[0];
      bb[0] = s1[1];	bb[1] = s2[1];	bb[2] = s3[1];
      cc[0] = s1[2];	cc[1] = s2[2];	cc[2] = s3[2];
      dd[0] = s1[3];	dd[1] = s2[3];	dd[2] = s3[3];
    }
  }

  /*** Compute the snow one scan at a time. ***/
  for(ThisScanNo=0; ThisScanNo<NoScans; ThisScanNo++) {

    /* Get values from dead and noisy detector lists. */
    int	ListIndex1 = (ThisScanNo%NUMBER_OF_DETECTORS) + BAND1_DETECTOR_OFFSET;
    int	ListIndex2 = (ThisScanNo%NUMBER_OF_DETECTORS) + BAND2_DETECTOR_OFFSET;
    int	ListIndex4 = (ThisScanNo%NUMBER_OF_DETECTORS) + BAND4_DETECTOR_OFFSET;
    int	ListIndex6 = (ThisScanNo%NUMBER_OF_DETECTORS) + BAND6_DETECTOR_OFFSET;
    NoisyDetectorFlag =
      (NoisyDetectorList[ListIndex1] != 0) || (NoisyDetectorList[ListIndex2] != 0) ||
      (NoisyDetectorList[ListIndex4] != 0) || (NoisyDetectorList[ListIndex6] != 0);
    ;
    DeadDetectorFlag =
      (DeadDetectorList[ListIndex1] != 0) || (DeadDetectorList[ListIndex2] != 0) ||
      (DeadDetectorList[ListIndex4] != 0) || (DeadDetectorList[ListIndex6] != 0);

    ProcessGeoFile( READ );
    ProcessL1Bfile( READ );
    ComputeSfcTemp( READ );
    ProcessCloudFile( READ );
    ComputeSnow();
    ComputeFractionalSnow();
    ProcessSnowFile( WRITE );
  }

  ProcessCloudFile( CLOSE );
  ProcessGeoFile( CLOSE );
  ComputeSfcTemp( CLOSE );
  ProcessL1Bfile( CLOSE );

  /*** Get the nuymber of daytime land pixels. ***/
  cnt.LandDayPixels = cnt.Snow_Snow + cnt.Snow_CloudObscured +
    cnt.Snow_Lake + cnt.Snow_LakeIce + cnt.Snow_NoSnow;

  ProcessSnowFile( CLOSE );

  MakeMeta();			/* Generate remaining PGE meta data */


  /*** The RCS identification strings have now all been put in the RCSidentString.
       Now write this out as meta data. ***/
  if( SDsetattr( SnowHDFhandle,"SCF Algorithm Version", DFNT_CHAR,
		 strlen(RCSidentString), RCSidentString ) != PGS_S_SUCCESS )
    HALT2(ACT02,"Can't SDsetattr for the program's RCS version strings");


  /*** Write out the surface temperature threshold.  ***/
  sprintf( HoldValue, "%.1f", MAX_SFC_TEMP_FOR_SNOW );
  if( SDsetattr( SnowHDFhandle,"Surface_Temperature_Screen_Threshold", DFNT_CHAR,
		 strlen(HoldValue), HoldValue ) != PGS_S_SUCCESS )
    HALT2(ACT02,"Can't SDsetattr for Surface_Temperature_Screen_Threshold");


  /*** Write out the geolocation offsets. ***/
  HoldFloat32 = 0.5;
  if( SDsetattr( SnowHDFhandle,
		 "HDFEOS_FractionalOffset_Along_swath_lines_500m_MOD_Swath_Snow",
		 DFNT_FLOAT32, 1, &HoldFloat32 ) != PGS_S_SUCCESS )
    HALT2(ACT02,"Can't SDsetattr for HDFEOS_FractionalOffset_Along_swath_lines");

  HoldFloat32 = 0.0;
  if( SDsetattr( SnowHDFhandle,
		 "HDFEOS_FractionalOffset_Cross_swath_pixels_500m_MOD_Swath_Snow",
		 DFNT_FLOAT32, 1, &HoldFloat32 ) != PGS_S_SUCCESS )
    HALT2(ACT02,"Can't SDsetattr for HDFEOS_FractionalOffset_Cross_swath_pixels");


  /*** Write out the meta data to the snow output file. ***/
  if(PGS_MET_Write(Meta500Handles[1], CORE, SnowHDFhandle) != PGS_S_SUCCESS)
    HALT2(ACT05,"PGS_MET_Write -- Error writing CORE");

  if(PGS_MET_Write(Meta500Handles[2], ARCHIVE, SnowHDFhandle) != PGS_S_SUCCESS)
    HALT2(ACT05,"PGS_MET_Write -- Error writing ARCHIVE");

  PGS_MET_Remove();

  ProcessSnowFile( TERMINATE );


  /*** Make the 5 km snow product. ***/
  Process5kmSnowFile( WRITE );

  Process5kmSnowFile( CLOSE );

  Process5kmSnowFile( TERMINATE );

#ifdef DEBUGGING
  fprintf(stderr, "FILL_VALUE........... %8d\n", cnt.Snow_FillValue  );
  fprintf(stderr, "DETECTOR_SATURATED... %8d\n", cnt.Snow_DetectorSaturated  );
  fprintf(stderr, "SNOW................. %8d\n", cnt.Snow_Snow  );
  fprintf(stderr, "CLOUD_OBSCURED....... %8d\n", cnt.Snow_CloudObscured  );
  fprintf(stderr, "OCEAN................ %8d\n", cnt.Snow_Ocean  );
  fprintf(stderr, "LAKE................. %8d\n", cnt.Snow_Lake  );
  fprintf(stderr, "LAKE_ICE............. %8d\n", cnt.Snow_LakeIce  );
  fprintf(stderr, "NO_SNOW.............. %8d\n", cnt.Snow_NoSnow  );
  fprintf(stderr, "NIGHT................ %8d\n", cnt.Snow_Night  );
  fprintf(stderr, "NO_DECISION.......... %8d\n", cnt.Snow_NoDecision  );
  fprintf(stderr, "L1B_MISSING_DATA..... %8d\n\n", cnt.Snow_L1BMissingData  );

  fprintf(stderr, "Total of above snow.. %8d\n\n",
	  cnt.Snow_FillValue +
	  cnt.Snow_DetectorSaturated +
	  cnt.Snow_Snow +
	  cnt.Snow_CloudObscured +
	  cnt.Snow_Ocean +
	  cnt.Snow_Lake +
	  cnt.Snow_LakeIce +
	  cnt.Snow_NoSnow +
	  cnt.Snow_Night +
	  cnt.Snow_NoDecision +
	  cnt.Snow_L1BMissingData );

  fprintf(stderr, "AllPixelsProcessed.............. %8d\n", cnt.AllPixelsProcessed  );
  fprintf(stderr, "InvalidInput.................... %8d\n", cnt.InvalidInput  );
  fprintf(stderr, "LandDayPixels................... %8d\n", cnt.LandDayPixels  );
  fprintf(stderr, "LandPixels...................... %8d\n", cnt.LandPixels  );
  fprintf(stderr, "NightPixels..................... %8d\n", cnt.NightPixels  );
  fprintf(stderr, "NoDecision...................... %8d\n", cnt.NoDecision  );

  fprintf(stderr, "SnowRejectedBySurfaceTemp....... %8d\n\n", cnt.Snow_Rejected_Warm  );

  fprintf(stderr, "QA_Other........................ %8d\n", cnt.QA_Other );
  fprintf(stderr, "QA_Good......................... %8d\n", cnt.QA_Good );
  fprintf(stderr, "QA_Ocean........................ %8d\n", cnt.QA_Ocean );

  fprintf(stderr, "\n**************************\n" );
  fprintf(stderr,   "FRACTIONAL SNOW STATISTICS\n" );
  fprintf(stderr,   "**************************\n\n");
  fprintf(stderr, "LandPixels...................... %8d\n", cnt.LandPixels  );
  fprintf(stderr, "NightPixels..................... %8d\n", cnt.NightPixels  );
  fprintf(stderr, "NoDecision...................... %8d\n", cnt.NoDecision  );
  fprintf(stderr, "SnowRejectedBySurfaceTemp....... %8d\n\n", cnt.Snow_Rejected_Warm  );
  fprintf(stderr,"Fractional snow normal......... %8d\n", FracNormal );
  fprintf(stderr,"Fractional snow big............ %8d\n", FracBig );
  fprintf(stderr,"Fractional snow little......... %8d\n", FracLittle );
#endif

  return(0);
}
