/* $Header: /home/powell/mod10/TERRA/v5.0.1/RCS/MOD_PR10_Compute_FractionalSnow.c,v 1.2 2005/05/19 18:13:07 powell Exp $

***************************************************************************************
!C

!Description:
	This function is responsible for computing fractional snow cover from the
	reflectances from cloud unobstructed MODIS bands 2, 4, and 6.  Various validity
	checks are performed.  Each call to this function processes one scan.

Program returns:
	No return code if successful.  If a failure, program halts with the reason
	given in a call to PGS_SMF_GenerateStatusReport.

!Input Parameters:
	refl1_p		MODIS channel 1 reflectance scan pointer
	refl2_p		MODIS channel 2 reflectance scan pointer
	refl4_p		MODIS channel 4 reflectance scan pointer
	refl6or7_p	MODIS channel 6 reflectance scan pointer
	CloudMask	Cloud flags array
	LandWater_p	Land/water mask scan pointer

!Output Parameters:
	FracSnow_p		Snow cover scan pointer

!Revision History:

   $Log: MOD_PR10_Compute_FractionalSnow.c,v $
   Revision 1.2  2005/05/19 18:13:07  powell
   Fix phone number.

   Revision 1.1  2005/05/19 17:37:08  powell
   Initial revision


!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

    Portions developed at the National Center for Supercomputing
    Applications at the Univ. of Illinois at Urbana-Champaign.

   Principal Investigator, Vincent Salomonson
        Vincent.V.Salomonson@NASA.gov
   Associate Investigator, Dorothy Hall
        Dorothy.K.Hall@NASA.gov
   Scientist/Programmer George Riggs
        George.A.Riggs.1@gsfc.nasa.gov
        301-614-5650
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706
   Programmer, Hugh W. Powell
        Hugh.W.Powell.1@gsfc.nasa.gov
        301-286-2310
        NASA/GSFC/Code 614.1
   Programmer, Nicolo E. DiGirolamo
        Nicolo.E.DiGirolamo.1.1@gsfc.nasa.gov
        301-614-5824
        NASA/GSFC/Code 614.0
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706

!References and Credits:
	______________________________________________________________________________

!Design Notes:
        This is the Collection 5 version of the code.
!END
***************************************************************************************/

#include <stdio.h>
#include <string.h>
#include "hdfi.h"
#include "PGS_IO.h"
#include "PGS_MEM.h"
#include "PGS_MET.h"
#include "MOD_PR10.h"

/*** Possible values for fractional snow_cover SDS. ***/
#define L1B_MISSING_DATA_F	200	/** 255 **/
#define DETECTOR_SATURATED_F	254	/** 254 **/
#define NO_DECISION_F		201	/** 253 **/
#define NIGHT_F			211	/** 252 **/
#define CLOUD_OBSCURED_F	250	/** 251 **/
#define OCEAN_F			239	/** 250 **/
#define LAKE_F			237	/** 249 **/
/** #define LAKE_ICE_F		248	Not Used.   **/
/** #define SNOW_F			166     247 **/
#define NO_SNOW_F		0	/** 246 **/

/*** Global Variables ***/
extern char	HaltMsg[];
extern uint8	*cloud_p, CloudMask[MAX_SAMPS];

/*** The column and row dimensions of the L1B earth view input channels. ***/
/*** The present scan (line) number being processed. ***/
extern int32	NoSamps;

/*** These pointers are set to the beginning of the scan line before this function
  is called.  They are incremented as the scan line is processed during the execution
  of this routine. ***/
extern float32	*refl1_p, *refl2_p;		/* MODIS reflectances */
extern float32	*refl4_p, *refl6or7_p;		/* MODIS reflectances */

/* for Surface Temp */
extern float32 *SfcTemp_p;                     /* MODIS-derived Surface Temperature*/

extern uint8	*FracSnow_p;			/* FractionalSnowCover output */

extern uint8	*LandWater_p;			/* Land/water mask */
extern char	RCSidentString[];

extern struct COUNTS	cntf;
int		FracBig, FracNormal, FracLittle;

/*====================================================================================*/

void ComputeFractionalSnow()
{
  char *rcs="$Id: MOD_PR10_Compute_FractionalSnow.c,v 1.2 2005/05/19 18:13:07 powell Exp $";

  int32	j;
  float32	ndsi, FracSnow;
  static char	FirstTimeFlag = XTRUEX;
  static struct COUNTS	cntf;


  static uint8	FractionalSnowCover[MAX_SAMPS];

  memset(FractionalSnowCover, L1B_MISSING_DATA_F, MAX_SAMPS);

  FracSnow_p   = FractionalSnowCover;


  /*** Add the RCS identification to the global string. ***/
  if( FirstTimeFlag == XTRUEX ) {
    FirstTimeFlag = XFALSEX;
    strcat( RCSidentString, "\t" );
    strcat( RCSidentString, rcs );
    strcat( RCSidentString, "\n" );
  }


  /*** Check L1B_FILL value. If the first pixel of a scan is this value, then
       the entire scan is the same.  Avoid unnecessary processing for this scan. ***/
  if( (*refl1_p == L1B_FILL) || (*refl2_p == L1B_FILL) ||
      (*refl4_p == L1B_FILL) || (*refl6or7_p == L1B_FILL) ){
    /*** FractionalSnowCover already defaults to L1B_MISSING_DATA. ***/
    cntf.QA_MissingData += NoSamps;
    cntf.AllPixelsProcessed += NoSamps;
    return;	     /*** Exit the routine.  No further processing needed. ***/
  }


  /*** Process each pixel for this scan. ***/

  for( j=0, FracSnow_p=FractionalSnowCover;
       j<NoSamps; j++, refl1_p++, refl2_p++, refl4_p++, refl6or7_p++, cloud_p++,
	 SfcTemp_p++, LandWater_p++, FracSnow_p++ ) {

    cntf.AllPixelsProcessed ++;

    /*** Determine if this pixel is to be considered land or water. ***/
    if( *LandWater_p == DEEP_OCEAN		||
	*LandWater_p == CONTINENTAL_SHELF_OCEAN ||
	*LandWater_p == SHALLOW_OCEAN 	) {
      *FracSnow_p = OCEAN_F;		/* Come here for OCEAN */
      cntf.Snow_Ocean ++;
      continue;			/* It's ocean, go to bottom of loop. */
    }

    /*** Be sure the L1B input values are usable.  Check all four channels.
	 The snow value will indicate a reason for one of the channels but
	 it will not indicate which channel has the problem; it could be
	 more than one channel and they could have different problems. ***/
    if( (*refl1_p > MAX_USABLE_L1B) || (*refl2_p > MAX_USABLE_L1B) ||
	(*refl4_p > MAX_USABLE_L1B) || (*refl6or7_p > MAX_USABLE_L1B) ) {

      /*** Check for saturated detector. ***/
      if( (*refl1_p == SATURATED_DETECTOR) || (*refl2_p == SATURATED_DETECTOR) ||
	  (*refl4_p == SATURATED_DETECTOR) || (*refl6or7_p == SATURATED_DETECTOR) ) {
	*FracSnow_p = DETECTOR_SATURATED_F;
	cntf.Snow_DetectorSaturated ++;
	continue;
      }

      /*** Check for L1B missing. ***/
      if( (*refl1_p == L1B_MISSING) || (*refl2_p == L1B_MISSING) ||
	  (*refl4_p == L1B_MISSING) || (*refl6or7_p == L1B_MISSING) ) {
	*FracSnow_p = L1B_MISSING_DATA_F;
	cntf.QA_MissingData ++;
	continue;
      }

      /*** It is unusable for some other reason. ***/
      *FracSnow_p = NO_DECISION_F;
      cntf.NoDecision ++;
      continue;
    }



    /*** Come here if this pixel is LAND, COASTLINES, SHALLOW_INLAND_WATER,
	 EPHEMERAL_WATER, DEEP_INLAND_WATER. ***/
    cntf.LandPixels ++;


    /*** Don't compute snow unless the cloud mask can be determined. ***/
    if( ! (*cloud_p & CLOUD_MASK_CF) ) {
      *FracSnow_p = NO_DECISION_F;
      cntf.NoDecision ++;
      continue;
    }

    /*** Come here if all the L1B input channels have valid values. ***/

    /*** Catching division by zero.  If the denominator of the NDSI is zero then a
	 value will be assigned to that pixel and the NDSI calculation will be skipped,
	 otherwise the NDSI will be calculated. ***/
    if( (*refl4_p + *refl6or7_p) > FLT_MIN)
      ndsi = (*refl4_p - *refl6or7_p) / (*refl4_p + *refl6or7_p);
    else
      ndsi = 0.0;



    /*** At this point this pixel's land/water mask has one of the following
	 possible values:   LAND, COASTLINES, SHALLOW_INLAND_WATER, EPHEMERAL_WATER,
	 DEEP_INLAND_WATER ***/

    if( *LandWater_p == SHALLOW_INLAND_WATER || *LandWater_p == DEEP_INLAND_WATER ) {

      if( *refl2_p < 0.11  ||  *refl4_p < 0.10) {
	*FracSnow_p = LAKE_F;

      } else {

	if( ndsi < 0 ) {
	  FracLittle ++;
	  *FracSnow_p = LAKE_F;

	} else {

	  FracSnow = -0.01 + 1.45 * ndsi;
	  if( FracSnow < 0 )
	    FracSnow = 0;

	  if( FracSnow > 1.0 )
	    FracBig ++;
	  else
	    FracNormal ++;

	  *FracSnow_p = FracSnow * 100;
	  if( *FracSnow_p > 100 )
	    *FracSnow_p = 100;

	  /* Check Surface Temp */
	  if( *SfcTemp_p > MAX_SFC_TEMP_FOR_SNOW ) {
	    *FracSnow_p = LAKE_F;
	    cntf.Snow_Rejected_Warm++;
	  }
	}

      }    /*** End of SHALLOW_INLAND_WATER and DEEP_INLAND_WATER block.   ***/

    } else {

      /*** Come here for LAND, COASTLINES, EPHEMERAL_WATER.  Determine if this pixel
	   is SNOW, SNOW_FOREST, or NO_SNOW.  Two snow regions (grouped criteria
	   tests) are used, snow anywhere and snow in forests. ***/

      if( (ndsi<0) || (*refl2_p<0.11) || (*refl4_p<0.10) ) {
	FracLittle ++;
	*FracSnow_p = NO_SNOW_F;

      } else {
	FracSnow = -0.01 + 1.45 * ndsi;
	if( FracSnow < 0 )
	  FracSnow = 0;

	if( FracSnow > 1.0 )
	  FracBig ++;
	else
	  FracNormal ++;

	*FracSnow_p = FracSnow * 100;
	if( *FracSnow_p > 100 )
	  *FracSnow_p = 100;
      }

      /* Check Surface Temp */
      if( *SfcTemp_p > MAX_SFC_TEMP_FOR_SNOW ) {
	*FracSnow_p = NO_SNOW_F;
	cntf.Snow_Rejected_Warm++;
      }

    }		/*** End of LAND, COASTLINES, EPHEMERAL_WATER block.   ***/

    /*** Only daytime samples are to be processed, see if this pixel is night. ***/
    if( ! (*cloud_p & CLOUD_MASK_3_CF) ) {
      *FracSnow_p = NIGHT_F;
      cntf.NightPixels ++;
      continue;		/* L1B data is not daytime, go to bottom of loop. */
    }

    /*** The snow for this pixel has been determined without regard to cloudiness. ***/



    /*** Come here to overlay the snow with STANDARD CLOUD TEST. ***/
    /*** Check cloud obstruction bits:
	 00 = cloud			reject it
	 01 = 66% probability of clear	use it
	 10 = 95% probability of clear	use it
	 11 = 99% probability of clear	use it	*/
    if( ! ( (*cloud_p & (CLOUD_MASK_1_CF | CLOUD_MASK_2_CF)) ) ) {
      *FracSnow_p = CLOUD_OBSCURED_F;
      cntf.Snow_CloudObscured ++;
    }


  }		/*** End: for( j=0.... ***/

  FracSnow_p = FractionalSnowCover;
  cloud_p = CloudMask;
}
