/* $Header: /home/powell/mod10/TERRA/v5.0.1/RCS/MOD_PR10_Compute_SfcTemp.c,v 1.2 2005/05/19 18:15:12 powell Exp $

***************************************************************************************
!C

!Description:
	This function is responsible for obtaining one scan line of the MODIS
	emissivities from the L1B file, and computing an estimated surface temperature.

Program returns:
	No return code if successful.  If a failure, program halts with the reason given
	in a call to PGS_SMF_GenerateStatusReport.

!Input Parameters:
	L1B1KM_INPUT	The PCF number for the L1B input file name
	CallType	Operation to be performed (OPEN,READ,CLOSE)

!Output Parameters:
        SfcTemp_p       Pointer to MODIS Surface Temperature, derived from
			channels 31 and 32

!Revision History:

   $Log: MOD_PR10_Compute_SfcTemp.c,v $
   Revision 1.2  2005/05/19 18:15:12  powell
   Fix phone number.

   Revision 1.1  2005/05/19 17:38:38  powell
   Initial revision


!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

    Portions developed at the National Center for Supercomputing
    Applications at the Univ. of Illinois at Urbana-Champaign.

   Principal Investigator, Vincent Salomonson
        Vincent.V.Salomonson@NASA.gov
   Associate Investigator, Dorothy Hall
        Dorothy.K.Hall@NASA.gov
   Scientist/Programmer George Riggs
        George.A.Riggs.1@gsfc.nasa.gov
        301-614-5650
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706
   Programmer, Hugh W. Powell
        Hugh.W.Powell.1@gsfc.nasa.gov
        301-286-2310
        NASA/GSFC/Code 614.1
   Programmer, Nicolo E. DiGirolamo
        Nicolo.E.DiGirolamo.1.1@gsfc.nasa.gov
        301-614-5824
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706

!References and Credits:

!Design Notes:
	This is the Collection 5 version of the code.  This function performs three
	separate functions, as indicated by the CallType input parameter.  It OPENs the
	L1B file at the beginning of the program.  It READs the two MODIS channels,
	applies the scale factor and offset, and converts from integer to floating
	point.  Each READ call processes one scan.  It CLOSEs the file at the conclusion
	of the program.

!END
***************************************************************************************/

#include <stdio.h>
#include <math.h>
#include "PGS_IO.h"
#include "PGS_MEM.h"
#include "PGS_MET.h"
#include "hdf.h"
#include "mfhdf.h"
#include "HdfEosDef.h"
#include "MOD_PR10.h"
#include "MOD_PR10_ActionMessages.h"

#define MODIS_CHAN31_INDEX	10
#define MODIS_CHAN32_INDEX	11

#define CW31			(float64)1.103e-5
#define CW32			(float64)1.202e-5
#define CW31_5			(CW31*CW31*CW31*CW31*CW31)
#define CW32_5			(CW32*CW32*CW32*CW32*CW32)

/*** Global Variables ***/
extern float32		ValidEVobservations[38], SaturatedEVobservations[38];
extern float64	aa[], bb[], cc[], dd[];
extern int8		DeadDetectorList[490], NoisyDetectorList[490];
extern char		in1km_FileName[PGSd_PC_FILE_PATH_MAX];

extern char	HaltMsg[];
extern enum DayNightType DayNightBothFlag;
extern struct COUNTS	cnt;
extern struct CHECK	ck;

/*** The column and row dimensions of the L1B earth view input channels. ***/
/*** The present scan (line) number being processed. ***/
extern int32	NoSamps, NoScans, ThisScanNo;
extern int32	No1kmSamps, No1kmScans;

/*** Pointers to the MODIS Surface Temperature, Degrees K. ***/
float32         *SfcTemp_p;
extern char	RCSidentString[];

/*====================================================================================*/

void ComputeSfcTemp( enum OpType CallType )
{
  char *rcs="$Id: MOD_PR10_Compute_SfcTemp.c,v 1.2 2005/05/19 18:15:12 powell Exp $";

  static float64 secantThetaMinusOne[MAX_SAMPS] = {
    0.743447, 0.739925, 0.736421, 0.732935, 0.729466, 0.726014, 0.722580, 0.719163,
    0.715763, 0.712379, 0.709012, 0.705662, 0.702329, 0.699012, 0.695711, 0.692426,
    0.689157, 0.685905, 0.682668, 0.679447, 0.676242, 0.673052, 0.669878, 0.666719,
    0.663576, 0.660447, 0.657334, 0.654236, 0.651152, 0.648083, 0.645030, 0.641990,
    0.638965, 0.635955, 0.632959, 0.629977, 0.627009, 0.624055, 0.621115, 0.618189,
    0.615277, 0.612378, 0.609494, 0.606622, 0.603764, 0.600920, 0.598089, 0.595271,
    0.592466, 0.589674, 0.586895, 0.584129, 0.581375, 0.578635, 0.575907, 0.573192,
    0.570489, 0.567798, 0.565120, 0.562455, 0.559801, 0.557159, 0.554530, 0.551913,
    0.549307, 0.546713, 0.544132, 0.541561, 0.539003, 0.536456, 0.533920, 0.531396,
    0.528883, 0.526382, 0.523892, 0.521413, 0.518945, 0.516488, 0.514042, 0.511607,
    0.509182, 0.506769, 0.504366, 0.501974, 0.499593, 0.497222, 0.494862, 0.492512,
    0.490172, 0.487843, 0.485524, 0.483215, 0.480917, 0.478628, 0.476349, 0.474081,
    0.471822, 0.469573, 0.467334, 0.465105, 0.462885, 0.460675, 0.458475, 0.456284,
    0.454103, 0.451931, 0.449769, 0.447616, 0.445472, 0.443337, 0.441212, 0.439096,
    0.436988, 0.434890, 0.432801, 0.430721, 0.428650, 0.426587, 0.424534, 0.422489,
    0.420452, 0.418425, 0.416406, 0.414396, 0.412394, 0.410401, 0.408416, 0.406440,
    0.404472, 0.402512, 0.400560, 0.398617, 0.396682, 0.394755, 0.392836, 0.390926,
    0.389023, 0.387128, 0.385241, 0.383363, 0.381492, 0.379628, 0.377773, 0.375925,
    0.374085, 0.372253, 0.370428, 0.368611, 0.366802, 0.365000, 0.363205, 0.361418,
    0.359639, 0.357866, 0.356101, 0.354344, 0.352593, 0.350850, 0.349114, 0.347386,
    0.345664, 0.343949, 0.342242, 0.340541, 0.338848, 0.337161, 0.335482, 0.333809,
    0.332143, 0.330484, 0.328832, 0.327187, 0.325548, 0.323916, 0.322290, 0.320672,
    0.319060, 0.317454, 0.315855, 0.314263, 0.312677, 0.311097, 0.309524, 0.307957,
    0.306397, 0.304843, 0.303295, 0.301754, 0.300219, 0.298690, 0.297167, 0.295650,
    0.294140, 0.292636, 0.291137, 0.289645, 0.288159, 0.286679, 0.285205, 0.283737,
    0.282274, 0.280818, 0.279367, 0.277923, 0.276484, 0.275051, 0.273624, 0.272202,
    0.270786, 0.269376, 0.267972, 0.266573, 0.265180, 0.263792, 0.262410, 0.261034,
    0.259663, 0.258298, 0.256938, 0.255583, 0.254234, 0.252891, 0.251552, 0.250220,
    0.248892, 0.247570, 0.246253, 0.244941, 0.243635, 0.242334, 0.241038, 0.239747,
    0.238462, 0.237181, 0.235906, 0.234636, 0.233371, 0.232111, 0.230856, 0.229606,
    0.228361, 0.227121, 0.225886, 0.224656, 0.223431, 0.222210, 0.220995, 0.219785,
    0.218579, 0.217378, 0.216182, 0.214991, 0.213804, 0.212623, 0.211446, 0.210274,
    0.209106, 0.207943, 0.206785, 0.205631, 0.204482, 0.203338, 0.202198, 0.201063,
    0.199932, 0.198806, 0.197685, 0.196568, 0.195455, 0.194347, 0.193243, 0.192144,
    0.191049, 0.189959, 0.188872, 0.187791, 0.186713, 0.185640, 0.184572, 0.183507,
    0.182447, 0.181391, 0.180340, 0.179292, 0.178249, 0.177210, 0.176176, 0.175145,
    0.174119, 0.173097, 0.172078, 0.171064, 0.170055, 0.169049, 0.168047, 0.167049,
    0.166056, 0.165066, 0.164081, 0.163099, 0.162122, 0.161148, 0.160178, 0.159213,
    0.158251, 0.157293, 0.156339, 0.155389, 0.154443, 0.153501, 0.152562, 0.151628,
    0.150697, 0.149770, 0.148847, 0.147927, 0.147012, 0.146100, 0.145192, 0.144287,
    0.143387, 0.142490, 0.141597, 0.140707, 0.139821, 0.138939, 0.138061, 0.137186,
    0.136314, 0.135447, 0.134583, 0.133722, 0.132865, 0.132012, 0.131162, 0.130316,
    0.129473, 0.128634, 0.127798, 0.126966, 0.126138, 0.125313, 0.124491, 0.123672,
    0.122858, 0.122046, 0.121238, 0.120434, 0.119633, 0.118835, 0.118041, 0.117250,
    0.116462, 0.115678, 0.114897, 0.114119, 0.113345, 0.112574, 0.111806, 0.111042,
    0.110281, 0.109523, 0.108768, 0.108017, 0.107269, 0.106524, 0.105783, 0.105044,
    0.104309, 0.103577, 0.102848, 0.102123, 0.101400, 0.100681, 0.099965, 0.099252,
    0.098542, 0.097835, 0.097131, 0.096431, 0.095733, 0.095039, 0.094348, 0.093659,
    0.092974, 0.092292, 0.091613, 0.090937, 0.090264, 0.089594, 0.088927, 0.088263,
    0.087602, 0.086944, 0.086289, 0.085637, 0.084988, 0.084342, 0.083699, 0.083058,
    0.082421, 0.081787, 0.081155, 0.080527, 0.079901, 0.079278, 0.078658, 0.078041,
    0.077427, 0.076816, 0.076208, 0.075602, 0.074999, 0.074399, 0.073802, 0.073208,
    0.072617, 0.072028, 0.071442, 0.070859, 0.070279, 0.069701, 0.069127, 0.068555,
    0.067986, 0.067419, 0.066855, 0.066294, 0.065736, 0.065181, 0.064628, 0.064078,
    0.063531, 0.062986, 0.062444, 0.061905, 0.061368, 0.060834, 0.060303, 0.059774,
    0.059248, 0.058725, 0.058204, 0.057686, 0.057171, 0.056658, 0.056148, 0.055641,
    0.055136, 0.054633, 0.054134, 0.053637, 0.053142, 0.052650, 0.052161, 0.051674,
    0.051190, 0.050708, 0.050229, 0.049752, 0.049278, 0.048807, 0.048337, 0.047871,
    0.047407, 0.046946, 0.046487, 0.046030, 0.045576, 0.045125, 0.044676, 0.044230,
    0.043786, 0.043344, 0.042905, 0.042469, 0.042034, 0.041603, 0.041173, 0.040747,
    0.040322, 0.039901, 0.039481, 0.039064, 0.038649, 0.038237, 0.037827, 0.037420,
    0.037015, 0.036613, 0.036212, 0.035815, 0.035419, 0.035026, 0.034636, 0.034247,
    0.033862, 0.033478, 0.033097, 0.032718, 0.032342, 0.031968, 0.031596, 0.031226,
    0.030859, 0.030494, 0.030132, 0.029772, 0.029414, 0.029059, 0.028706, 0.028355,
    0.028006, 0.027660, 0.027316, 0.026974, 0.026635, 0.026298, 0.025963, 0.025631,
    0.025301, 0.024972, 0.024647, 0.024323, 0.024002, 0.023683, 0.023367, 0.023052,
    0.022740, 0.022430, 0.022123, 0.021817, 0.021514, 0.021213, 0.020914, 0.020618,
    0.020324, 0.020032, 0.019742, 0.019454, 0.019169, 0.018885, 0.018605, 0.018326,
    0.018049, 0.017775, 0.017503, 0.017233, 0.016965, 0.016699, 0.016436, 0.016174,
    0.015915, 0.015658, 0.015404, 0.015151, 0.014901, 0.014652, 0.014406, 0.014162,
    0.013920, 0.013681, 0.013443, 0.013208, 0.012975, 0.012744, 0.012515, 0.012288,
    0.012063, 0.011841, 0.011621, 0.011402, 0.011186, 0.010972, 0.010760, 0.010550,
    0.010343, 0.010137, 0.009934, 0.009733, 0.009534, 0.009336, 0.009141, 0.008949,
    0.008758, 0.008569, 0.008383, 0.008198, 0.008016, 0.007836, 0.007657, 0.007481,
    0.007307, 0.007135, 0.006965, 0.006798, 0.006632, 0.006468, 0.006307, 0.006147,
    0.005990, 0.005834, 0.005681, 0.005530, 0.005381, 0.005234, 0.005089, 0.004946,
    0.004805, 0.004666, 0.004530, 0.004395, 0.004262, 0.004132, 0.004003, 0.003877,
    0.003752, 0.003630, 0.003510, 0.003391, 0.003275, 0.003161, 0.003049, 0.002939,
    0.002830, 0.002724, 0.002620, 0.002518, 0.002418, 0.002321, 0.002225, 0.002131,
    0.002039, 0.001949, 0.001862, 0.001776, 0.001692, 0.001611, 0.001531, 0.001453,
    0.001378, 0.001304, 0.001233, 0.001163, 0.001096, 0.001030, 0.000967, 0.000905,
    0.000846, 0.000789, 0.000733, 0.000680, 0.000629, 0.000579, 0.000532, 0.000487,
    0.000443, 0.000402, 0.000363, 0.000326, 0.000291, 0.000257, 0.000226, 0.000197,
    0.000170, 0.000145, 0.000122, 0.000100, 0.000081, 0.000064, 0.000049, 0.000036,
    0.000025, 0.000016, 0.000009, 0.000004, 0.000001, 0.000001, 0.000004, 0.000009,
    0.000016, 0.000025, 0.000036, 0.000049, 0.000064, 0.000081, 0.000100, 0.000122,
    0.000145, 0.000170, 0.000197, 0.000226, 0.000257, 0.000291, 0.000326, 0.000363,
    0.000402, 0.000443, 0.000487, 0.000532, 0.000579, 0.000629, 0.000680, 0.000733,
    0.000789, 0.000846, 0.000905, 0.000967, 0.001030, 0.001096, 0.001163, 0.001233,
    0.001304, 0.001378, 0.001453, 0.001531, 0.001611, 0.001692, 0.001776, 0.001862,
    0.001949, 0.002039, 0.002131, 0.002225, 0.002321, 0.002418, 0.002518, 0.002620,
    0.002724, 0.002830, 0.002939, 0.003049, 0.003161, 0.003275, 0.003391, 0.003510,
    0.003630, 0.003752, 0.003877, 0.004003, 0.004132, 0.004262, 0.004395, 0.004530,
    0.004666, 0.004805, 0.004946, 0.005089, 0.005234, 0.005381, 0.005530, 0.005681,
    0.005834, 0.005990, 0.006147, 0.006307, 0.006468, 0.006632, 0.006798, 0.006965,
    0.007135, 0.007307, 0.007481, 0.007657, 0.007836, 0.008016, 0.008198, 0.008383,
    0.008569, 0.008758, 0.008949, 0.009141, 0.009336, 0.009534, 0.009733, 0.009934,
    0.010137, 0.010343, 0.010550, 0.010760, 0.010972, 0.011186, 0.011402, 0.011621,
    0.011841, 0.012063, 0.012288, 0.012515, 0.012744, 0.012975, 0.013208, 0.013443,
    0.013681, 0.013920, 0.014162, 0.014406, 0.014652, 0.014901, 0.015151, 0.015404,
    0.015658, 0.015915, 0.016174, 0.016436, 0.016699, 0.016965, 0.017233, 0.017503,
    0.017775, 0.018049, 0.018326, 0.018605, 0.018885, 0.019169, 0.019454, 0.019742,
    0.020032, 0.020324, 0.020618, 0.020914, 0.021213, 0.021514, 0.021817, 0.022123,
    0.022430, 0.022740, 0.023052, 0.023367, 0.023683, 0.024002, 0.024323, 0.024647,
    0.024972, 0.025301, 0.025631, 0.025963, 0.026298, 0.026635, 0.026974, 0.027316,
    0.027660, 0.028006, 0.028355, 0.028706, 0.029059, 0.029414, 0.029772, 0.030132,
    0.030494, 0.030859, 0.031226, 0.031596, 0.031968, 0.032342, 0.032718, 0.033097,
    0.033478, 0.033862, 0.034247, 0.034636, 0.035026, 0.035419, 0.035815, 0.036212,
    0.036613, 0.037015, 0.037420, 0.037827, 0.038237, 0.038649, 0.039064, 0.039481,
    0.039901, 0.040322, 0.040747, 0.041173, 0.041603, 0.042034, 0.042469, 0.042905,
    0.043344, 0.043786, 0.044230, 0.044676, 0.045125, 0.045576, 0.046030, 0.046487,
    0.046946, 0.047407, 0.047871, 0.048337, 0.048807, 0.049278, 0.049752, 0.050229,
    0.050708, 0.051190, 0.051674, 0.052161, 0.052650, 0.053142, 0.053637, 0.054134,
    0.054633, 0.055136, 0.055641, 0.056148, 0.056658, 0.057171, 0.057686, 0.058204,
    0.058725, 0.059248, 0.059774, 0.060303, 0.060834, 0.061368, 0.061905, 0.062444,
    0.062986, 0.063531, 0.064078, 0.064628, 0.065181, 0.065736, 0.066294, 0.066855,
    0.067419, 0.067986, 0.068555, 0.069127, 0.069701, 0.070279, 0.070859, 0.071442,
    0.072028, 0.072617, 0.073208, 0.073802, 0.074399, 0.074999, 0.075602, 0.076208,
    0.076816, 0.077427, 0.078041, 0.078658, 0.079278, 0.079901, 0.080527, 0.081155,
    0.081787, 0.082421, 0.083058, 0.083699, 0.084342, 0.084988, 0.085637, 0.086289,
    0.086944, 0.087602, 0.088263, 0.088927, 0.089594, 0.090264, 0.090937, 0.091613,
    0.092292, 0.092974, 0.093659, 0.094348, 0.095039, 0.095733, 0.096431, 0.097131,
    0.097835, 0.098542, 0.099252, 0.099965, 0.100681, 0.101400, 0.102123, 0.102848,
    0.103577, 0.104309, 0.105044, 0.105783, 0.106524, 0.107269, 0.108017, 0.108768,
    0.109523, 0.110281, 0.111042, 0.111806, 0.112574, 0.113345, 0.114119, 0.114897,
    0.115678, 0.116462, 0.117250, 0.118041, 0.118835, 0.119633, 0.120434, 0.121238,
    0.122046, 0.122858, 0.123672, 0.124491, 0.125313, 0.126138, 0.126966, 0.127798,
    0.128634, 0.129473, 0.130316, 0.131162, 0.132012, 0.132865, 0.133722, 0.134583,
    0.135447, 0.136314, 0.137186, 0.138061, 0.138939, 0.139821, 0.140707, 0.141597,
    0.142490, 0.143387, 0.144287, 0.145192, 0.146100, 0.147012, 0.147927, 0.148847,
    0.149770, 0.150697, 0.151628, 0.152562, 0.153501, 0.154443, 0.155389, 0.156339,
    0.157293, 0.158251, 0.159213, 0.160178, 0.161148, 0.162122, 0.163099, 0.164081,
    0.165066, 0.166056, 0.167049, 0.168047, 0.169049, 0.170055, 0.171064, 0.172078,
    0.173097, 0.174119, 0.175145, 0.176176, 0.177210, 0.178249, 0.179292, 0.180340,
    0.181391, 0.182447, 0.183507, 0.184572, 0.185640, 0.186713, 0.187791, 0.188872,
    0.189959, 0.191049, 0.192144, 0.193243, 0.194347, 0.195455, 0.196568, 0.197685,
    0.198806, 0.199932, 0.201063, 0.202198, 0.203338, 0.204482, 0.205631, 0.206785,
    0.207943, 0.209106, 0.210274, 0.211446, 0.212623, 0.213804, 0.214991, 0.216182,
    0.217378, 0.218579, 0.219785, 0.220995, 0.222210, 0.223431, 0.224656, 0.225886,
    0.227121, 0.228361, 0.229606, 0.230856, 0.232111, 0.233371, 0.234636, 0.235906,
    0.237181, 0.238462, 0.239747, 0.241038, 0.242334, 0.243635, 0.244941, 0.246253,
    0.247570, 0.248892, 0.250220, 0.251552, 0.252891, 0.254234, 0.255583, 0.256938,
    0.258298, 0.259663, 0.261034, 0.262410, 0.263792, 0.265180, 0.266573, 0.267972,
    0.269376, 0.270786, 0.272202, 0.273624, 0.275051, 0.276484, 0.277923, 0.279367,
    0.280818, 0.282274, 0.283737, 0.285205, 0.286679, 0.288159, 0.289645, 0.291137,
    0.292636, 0.294140, 0.295650, 0.297167, 0.298690, 0.300219, 0.301754, 0.303295,
    0.304843, 0.306397, 0.307957, 0.309524, 0.311097, 0.312677, 0.314263, 0.315855,
    0.317454, 0.319060, 0.320672, 0.322290, 0.323916, 0.325548, 0.327187, 0.328832,
    0.330484, 0.332143, 0.333809, 0.335482, 0.337161, 0.338848, 0.340541, 0.342242,
    0.343949, 0.345664, 0.347386, 0.349114, 0.350850, 0.352593, 0.354344, 0.356101,
    0.357866, 0.359639, 0.361418, 0.363205, 0.365000, 0.366802, 0.368611, 0.370428,
    0.372253, 0.374085, 0.375925, 0.377773, 0.379628, 0.381492, 0.383363, 0.385241,
    0.387128, 0.389023, 0.390926, 0.392836, 0.394755, 0.396682, 0.398617, 0.400560,
    0.402512, 0.404472, 0.406440, 0.408416, 0.410401, 0.412394, 0.414396, 0.416406,
    0.418425, 0.420452, 0.422489, 0.424534, 0.426587, 0.428650, 0.430721, 0.432801,
    0.434890, 0.436988, 0.439096, 0.441212, 0.443337, 0.445472, 0.447616, 0.449769,
    0.451931, 0.454103, 0.456284, 0.458475, 0.460675, 0.462885, 0.465105, 0.467334,
    0.469573, 0.471822, 0.474081, 0.476349, 0.478628, 0.480917, 0.483215, 0.485524,
    0.487843, 0.490172, 0.492512, 0.494862, 0.497222, 0.499593, 0.501974, 0.504366,
    0.506769, 0.509182, 0.511607, 0.514042, 0.516488, 0.518945, 0.521413, 0.523892,
    0.526382, 0.528883, 0.531396, 0.533920, 0.536456, 0.539003, 0.541561, 0.544132,
    0.546713, 0.549307, 0.551913, 0.554530, 0.557159, 0.559801, 0.562455, 0.565120,
    0.567798, 0.570489, 0.573192, 0.575907, 0.578635, 0.581375, 0.584129, 0.586895,
    0.589674, 0.592466, 0.595271, 0.598089, 0.600920, 0.603764, 0.606622, 0.609494,
    0.612378, 0.615277, 0.618189, 0.621115, 0.624055, 0.627009, 0.629977, 0.632959,
    0.635955, 0.638965, 0.641990, 0.645030, 0.648083, 0.651152, 0.654236, 0.657334,
    0.660447, 0.663576, 0.666719, 0.669878, 0.673052, 0.676242, 0.679447, 0.682668,
    0.685905, 0.689157, 0.692426, 0.695711, 0.699012, 0.702329, 0.705662, 0.709012,
    0.712379, 0.715763, 0.719163, 0.722580, 0.726014, 0.729466, 0.732935, 0.736421,
    0.739925, 0.743447 };


  float64	ln31, ln32, t31, t32;
  static float64	c1 = 1.1910439e-16;	/* 2*h*(c**2) = W m**(-2)	*/
  static float64	c2 = 1.4387686e-2;	/* (h*c)/k = m K		*/

  static int32		sd_id, sds_index1KM;
  static int32		SWfidin, SWidin;		/* Swath handles */

  /*** Pointers to the 2 MODIS reflectances, after scaling and offsets. ***/
  float32		*emiss31_p, *emiss32_p;

  /*** Scales and offsets used for converting the MODIS earth view channels. ***/
  static float32   scaleBand31,  scaleBand32;
  static float32   offsetBand31, offsetBand32;
  static int32	ThisScanNo_1km, ScanNowInBuffer = -999;

  /*----------------------------------------------------------------------------------*/

  if( CallType == READ ) {

    int16  j, k;

    static uint16	EV1KM_ch31[MAX_SAMPS/2],EV1KM_ch32[MAX_SAMPS/2];
    static float32	emiss_chan31[MAX_SAMPS],emiss_chan32[MAX_SAMPS];
    static float32      SfcTemp[MAX_SAMPS];
    static int32	start[3] = {9999,9999,0},  edge[3]  = {1,1,9999};


    edge[2] = NoSamps/2;		/*** Number of samples in this scan. ***/
    ThisScanNo_1km = ThisScanNo / 2;
    start[1] = ThisScanNo_1km;	/*** Which scan line to read ***/



    if( ThisScanNo_1km != ScanNowInBuffer ) {

      /*** Read in MODIS channel 31.  1 km ***/
      start[0] = MODIS_CHAN31_INDEX;		/* Subscript for MODIS channel 31 */
      if(SWreadfield(SWidin,"EV_1KM_Emissive", start, NULL, edge, EV1KM_ch31) == FAIL)
	HALT2(ACT06,"Failure reading EV_1KM MODIS channel 31");

      for(j=k=0; j<NoSamps; j+=2, k++) {
	if( EV1KM_ch31[k] <= MAX_USABLE_L1B ) {
	  emiss_chan31[j] =  emiss_chan31[j+1] =
	    ((float32)EV1KM_ch31[k]-offsetBand31) * scaleBand31;
	} else {
	  if( EV1KM_ch31[k] == 65535 )
	    emiss_chan31[j] = emiss_chan31[j+1] = -2;		/* Missing data */
	  else
	    emiss_chan31[j] = emiss_chan31[j+1] = -1;		/* Invalid value */
	}
      }


      /*** Read in MODIS channel 32.  1 km ***/
      start[0] = MODIS_CHAN32_INDEX;		/* Subscript for MODIS channel 32 */
      if(SWreadfield(SWidin,"EV_1KM_Emissive", start, NULL, edge, EV1KM_ch32) == FAIL)
	HALT2(ACT06,"Failure reading EV_1KM MODIS channel 32");

      for(j=k=0; j<NoSamps; j+=2, k++) {
	if( EV1KM_ch32[k] <= MAX_USABLE_L1B ) {
	  emiss_chan32[j] = emiss_chan32[j+1] =
	    ((float32)EV1KM_ch32[k]-offsetBand32) * scaleBand32;
	} else {
	  if( EV1KM_ch32[k] == 65535 )
	    emiss_chan32[j] = emiss_chan32[j+1] = -2;		/* Missing data */
	  else
	    emiss_chan32[j] = emiss_chan32[j+1] = -1;		/* Invalid value */
	}
      }
      ScanNowInBuffer = ThisScanNo_1km;
    } /* scan buffer check */

    /*** Set global pointers to the two MODIS scans. ***/
    emiss31_p = emiss_chan31;
    emiss32_p = emiss_chan32;

    /* Compute Surface Temp for this scan line */

    for(j=0; j<NoSamps; j++, emiss31_p++, emiss32_p++) {
      ln31 = CW31 * log((c1 / ( (CW31_5 * 1e6) * (float64)(*emiss31_p))) + 1.0 );
      t31 = c2 / ln31;

      ln32 = CW32 * log((c1 / ( (CW32_5 * 1e6) * (float64)(*emiss32_p))) + 1.0 );
      t32 = c2 / ln32;

      /*** Use the aa,bb,cc,dd coefs that were initialized in the main program. ***/
      if( t31 < 240 )
        SfcTemp[j]=aa[0] + bb[0]*t31 + cc[0]*(t31-t32) +
	  dd[0]*((t31-t32)*secantThetaMinusOne[j]);
      else if( t31 <= 260 )
        SfcTemp[j]=aa[1] + bb[1]*t31 + cc[1]*(t31-t32) +
	  dd[1]*((t31-t32)*secantThetaMinusOne[j]);
      else
        SfcTemp[j]=aa[2] + bb[2]*t31 + cc[2]*(t31-t32) +
	  dd[2]*((t31-t32)*secantThetaMinusOne[j]);
    }

    /*** Set global pointers ***/
    SfcTemp_p = SfcTemp;

    return;
  }
  /*----------------------------------------------------------------------------------*/

  if( CallType == OPEN ) {

    int32		attr_index, data_type, count;
    PGSt_integer	version;
    char		attr_name[64];
    void		*local;


    /*** Add the RCS identification to the global string. ***/
    strcat( RCSidentString, "\t" );
    strcat( RCSidentString, rcs );
    strcat( RCSidentString, "\n" );


    /*** Get GRingPoint's and RangeBeginning's.  These will be compared with those of
	 the other two input files to be sure that all files are the same granule. ***/
    version = 1;
    if(PGS_MET_GetPCAttr(L1B1KM_INPUT, version, CORE, "GRingPointLatitude.1",
			 ck.l1b.GRingPointLatitude ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting GRingPointLatitude.1");

    version = 1;
    if(PGS_MET_GetPCAttr(L1B1KM_INPUT, version, CORE, "GRingPointLongitude.1",
			 ck.l1b.GRingPointLongitude ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting GRingPointLongitude.1");

    local = ck.l1b.RangeBeginningDate;
    ck.l1b.RangeBeginningDate[0] = '\0';
    version = 1;
    if(PGS_MET_GetPCAttr(L1B1KM_INPUT, version, CORE, "RangeBeginningDate",
			 &local ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting RangeBeginningDate");

    local = ck.l1b.RangeBeginningTime;
    ck.l1b.RangeBeginningTime[0] = '\0';
    version = 1;
    if(PGS_MET_GetPCAttr(L1B1KM_INPUT, version, CORE, "RangeBeginningTime",
			 &local ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting RangeBeginningTime");

    /*** Open the L1B input file. ***/
    if( (sd_id = SDstart( in1km_FileName, DFACC_READ )) == FAIL )
      HALT2(ACT04,"Can't open the L1B input file.");


    /*** Get "Dead Detector List."  Values will be needed for all bands. ***/
    if( (attr_index = SDfindattr( sd_id, "Dead Detector List" )) == FAIL)
      HALT2(ACT09,"Can't get attr_index for 'Dead Detector List' in L1B file");

    if( SDattrinfo( sd_id, attr_index, attr_name, &data_type, &count ) == FAIL )
      HALT2(ACT09,"Can't get attrinfo for 'Dead Detector List' in L1B file");

    if( data_type != DFNT_INT8  || count != sizeof(DeadDetectorList) )
      HALT2(ACT09,"Dead Detector List is wrong type or wrong size");

    if( SDreadattr( sd_id, attr_index, DeadDetectorList ) == FAIL )
      HALT2(ACT09,"Can't read attribute for DeadDetectorList in L1B file");



    /*** Get "Noisy Detector List."  Values will be needed for all bands. ***/
    if( (attr_index = SDfindattr( sd_id, "Noisy Detector List" )) == FAIL)
      HALT2(ACT09,"Can't get attr_index for 'Noisy Detector List' in L1B file");

    if( SDattrinfo( sd_id, attr_index, attr_name, &data_type, &count ) == FAIL )
      HALT2(ACT09,"Can't get attrinfo for 'Noisy Detector List' in L1B file");

    if( data_type != DFNT_INT8  || count != sizeof(NoisyDetectorList) )
      HALT2(ACT09,"Noisy Detector List is wrong type or wrong size");

    if( SDreadattr( sd_id, attr_index, NoisyDetectorList ) == FAIL )
      HALT2(ACT09,"Can't read attribute for NoisyDetectorList in L1B file");



    /*** Get "%Valid EV Observations" for MODIS bands 2,4,6,31,32 (index 1,3,5,32,33).
	 These will be local attributes for the Snow Cover SDS. ***/
    if( (attr_index = SDfindattr( sd_id, "%Valid EV Observations" )) == FAIL)
      HALT2(ACT09,"Can't get attr_index for '%Valid EV Observations' in L1B file");

    if( SDattrinfo( sd_id, attr_index, attr_name, &data_type, &count ) == FAIL )
      HALT2(ACT09,"Can't get attrinfo for '%Valid EV Observations' in L1B file");

    if( data_type!=DFNT_FLOAT32 || count!=sizeof(ValidEVobservations) / sizeof(float32) )
      HALT2(ACT09,"%ValidEVobservations is wrong type or wrong size");

    if( SDreadattr( sd_id, attr_index, ValidEVobservations ) == FAIL )
      HALT2(ACT09,"Can't read attribute for '&Valid EV Observations' in L1B file");



    /*** Get "%Saturated EV Observations"for MODIS bands 2,4,6,31,32 (index 1,3,5,32,33).
	 These will be local attributes for the Snow Cover SDS. ***/
    if( (attr_index = SDfindattr( sd_id, "%Saturated EV Observations" )) == FAIL)
      HALT2(ACT09,"Can't get attr_index for '%Saturated EV Observations' in L1B file");

    if( SDattrinfo( sd_id, attr_index, attr_name, &data_type, &count ) == FAIL )
      HALT2(ACT09,"Can't get attrinfo for '%Saturated EV Observations' in L1B file");

    if(data_type!=DFNT_FLOAT32 || count!=sizeof(SaturatedEVobservations)/sizeof(float32))
      HALT2(ACT09,"%SaturatedEVobservations is wrong type or wrong size");

    if( SDreadattr( sd_id, attr_index, SaturatedEVobservations ) == FAIL )
      HALT2(ACT09,"Can't read attribute for '&Saturated EV Observations' in L1B file");


    /*** Get swath handle ***/
    if( (SWfidin = SWopen( in1km_FileName, DFACC_READ )) == FAIL )
      HALT2(ACT04,"SWopen failed for L1B file");

    if( (SWidin = SWattach( SWfidin, "MODIS_SWATH_Type_L1B" )) == FAIL )
      HALT2(ACT09,"SWattach failed for L1B file");

    if( SWdiminfo(SWidin, "Band_250M") != 2 )
      HALT2(ACT06,"Swath dimension for Band_250M is not 2");

    if( SWdiminfo(SWidin, "Band_500M") != 5 )
      HALT2(ACT06,"Swath dimension for Band_500M is not 5");

    if( (No1kmScans =  SWdiminfo(SWidin, "10*nscans")) == -1 )
      HALT2(ACT06,"Swath dimension for 10*nscans failed.");

    if( (No1kmSamps =  SWdiminfo(SWidin, "Max_EV_frames")) == -1 )
      HALT2(ACT06,"Swath dimension for Max_EV_frames failed.");

    if( No1kmSamps > MAX_SAMPS/2 )
      HALT2(ACT06,"2*Max_EV_Frames is greater than compiles value MAX_SAMPS/2");

    /*   AllPixelsProcessed = NoScans * NoSamps; */


    /*** Get 1 km emissive scales and offsets. ***/
    {
      char	*band1KMemissiveName = "EV_1KM_Emissive";
      float32	floatBuf[16];
      static int32   sds_id1KM,rank1KM,data_type1KM,dimsizes1KM[MAX_VAR_DIMS],nattrs1KM;

      if( (sds_index1KM = SDnametoindex( sd_id,band1KMemissiveName )) == FAIL )
	HALT2(ACT09,"sds_index1KM failed in L1B file");

      if( (sds_id1KM = SDselect( sd_id, sds_index1KM ) ) == FAIL )
	HALT2(ACT09,"sds_id1KM failed in L1B file");

      if(SDgetinfo( sds_id1KM, band1KMemissiveName, &rank1KM,
		    dimsizes1KM, &data_type1KM, &nattrs1KM) == FAIL )
	HALT2(ACT09,"SDgetinfo failed for band1KMemissiveName in L1B file");


      /*** Get 1KMm scales ***/
      if( (attr_index = SDfindattr( sds_id1KM, "radiance_scales" )) == FAIL)
	HALT2(ACT09,"Can't get attr_index for 1km 'radiance_scales' in L1B file");

      if( SDattrinfo( sds_id1KM, attr_index, attr_name, &data_type, &count ) == FAIL)
	HALT2(ACT09,"Can't get attrinfo for 1km 'radiance_scales' in L1B file");

      if( (data_type != DFNT_FLOAT32) || (count != (sizeof(floatBuf)/sizeof(float32))) )
	HALT2(ACT06,"sds_id1KM radiance_scales has bad data_type and/or count.");

      if( SDreadattr( sds_id1KM, attr_index, floatBuf ) == FAIL )
	HALT2(ACT09,"Can't read attribute for 1km 'radiance_scales' in L1B file");

      scaleBand31 = floatBuf[10];
      scaleBand32 = floatBuf[11];

      /*** Get 1KMm offsets ***/
      if( (attr_index = SDfindattr( sds_id1KM, "radiance_offsets" )) == FAIL)
	HALT2(ACT09,"Can't get attr_index for 1km 'radiance_offsets' in L1B file");

      if( SDattrinfo( sds_id1KM, attr_index, attr_name, &data_type, &count ) == FAIL )
	HALT2(ACT09,"Can't get attrinfo for 1km 'radiance_offsets' in L1B file");

      if( (data_type != DFNT_FLOAT32) || (count != (sizeof(floatBuf)/sizeof(float32))) )
	HALT2(ACT06,"sds_id1KM radiance_offsets has bad data_type and/or count.");

      if( SDreadattr( sds_id1KM, attr_index, floatBuf ) == FAIL )
	HALT2(ACT09,"Can't read attribute for 1km 'radiance_offsets' in L1B file");

      offsetBand31 = floatBuf[10];
      offsetBand32 = floatBuf[11];

      if( SDendaccess( sds_id1KM ) == FAIL )
	HALT2(ACT09,"Cannot SDendaccess for sds_id1KM");
    }
    return;
  }
  /*----------------------------------------------------------------------------------*/

  if( CallType == CLOSE ) {
    if( SWdetach( SWidin ) == FAIL )
      HALT2(ACT09,"Failure with SWdetach for L1B");

    if( SWclose( SWfidin ) == FAIL )
      HALT2(ACT09,"Failure with SWclose for L1B");

    if( SDend( sd_id ) == FAIL )
      HALT2(ACT09,"Failure with SDend for sd_id for L1B");
  }
}
