/* $Header: /home/powell/mod10/TERRA/v5.0.1/RCS/MOD_PR10_Compute_Snow.c,v 1.1 2005/05/19 17:30:23 powell Exp $

***************************************************************************************
!C

!Description:
	This function is responsible for computing snow cover from the reflectances from
	cloud unobstructed MODIS bands 2, 4, and 6.  Various validity checks are
	performed and QA statistics are maintained.  Each call to this function
	processes one scan.

Program returns:
	No return code if successful.  If a failure, program halts with the reason
	given in a call to PGS_SMF_GenerateStatusReport.

!Input Parameters:
	refl1_p		MODIS channel 1 reflectance scan pointer
	refl2_p		MODIS channel 2 reflectance scan pointer
	refl4_p		MODIS channel 4 reflectance scan pointer
	refl6or7_p	MODIS channel 6 or 7 reflectance scan pointer
	uncert6or7_p	MODIS channel 6 or 7 uncertainties
	CloudMask	Cloud flags array
	LandWater_p	Land/water mask scan pointer

!Output Parameters:
	snow_p		Snow cover scan pointer
	snowQA_p	Snow QA scan pointer

!Revision History:

   $Log: MOD_PR10_Compute_Snow.c,v $
   Revision 1.1  2005/05/19 17:30:23  powell
   Initial revision


!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

    Portions developed at the National Center for Supercomputing
    Applications at the Univ. of Illinois at Urbana-Champaign.

   Principal Investigator, Vincent Salomonson
        Vincent.V.Salomonson@NASA.gov
   Associate Investigator, Dorothy Hall
        Dorothy.K.Hall@NASA.gov
   Scientist/Programmer George Riggs
        George.A.Riggs.1@gsfc.nasa.gov
        301-614-5650
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706
   Programmer, Hugh W. Powell
        Hugh.W.Powell.1@gsfc.nasa.gov
        301-286-2310
        NASA/GSFC/Code 614.1
   Programmer, Nicolo E. DiGirolamo
        Nicolo.E.DiGirolamo.1.1@gsfc.nasa.gov
        301-614-5824
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706

!References and Credits:
	Major assistance with integrating PGS toolkit, HDF, and HDF-EOS
	was provided by Gang Ye of SDST/STIG.

!Design Notes:
        This is the Collection 5 version of the code. Output data arrays are; snow
	cover, fractional snow cover, snow spatial QA latitude and longitude.
	Fractional snow cover is new in Collection 5.  New to Collection 5 is that the
	snow spatial QA is written as a thematic QA as a byte value.  The spatial QA is
	not bit encoded as in past version but the QA checks of past versions are
	retained and used to set the QA value.  Calculation of a second snow map with a
	liberal cloud mask is not done in Collection 5.

	The presence or absence of snow is determined as a function of NDSI and MODIS
	channel 2 and 4 and by a screening of estimated surface temperature.  The
	fractional snow cover is calculated for all land and inland water pixels not
	obscured by cloud.

!END
***************************************************************************************/

#include <stdio.h>
#include <string.h>
#include "hdfi.h"
#include "PGS_IO.h"
#include "PGS_MEM.h"
#include "PGS_MET.h"
#include "MOD_PR10.h"

#define XX00	0.400
#define XX01	0.400
#define XX02	0.100
#define XX03	0.1125
#define XX04	0.125
#define XX05	0.175
#define XX06	0.200
#define XX07	0.250
#define XX08	0.275
#define XX09	0.325
#define XX10	0.400

#define YY00	1.0
#define YY01	0.1
#define YY02	0.24516133
#define YY03	0.29378870
#define YY04	0.34092437
#define YY05	0.51454990
#define YY06	0.59241240
#define YY07	0.73023683
#define YY08	0.79019879
#define YY09	0.89222209
#define YY10	1.000

/*** Global Variables ***/
extern char	HaltMsg[];
extern uint8	*cloud_p, CloudMask[MAX_SAMPS];

/*** The column and row dimensions of the L1B earth view input channels. ***/
/*** The present scan (line) number being processed. ***/
extern int32	NoSamps, NoScans, ThisScanNo;
extern int8	NoisyDetectorFlag, DeadDetectorFlag;

/*** These pointers are set to the beginning of the scan line before this function
  is called.  They are incremented as the scan line is processed during the execution
  of this routine. ***/
extern float32	*refl1_p, *refl2_p;		/* MODIS reflectances */
extern float32	*refl4_p, *refl6or7_p;		/* MODIS reflectances */

/* for Surface Temp */
extern float32 *SfcTemp_p;                     /* MODIS-derived Surface Temperature*/

extern uint8	*uncert6or7_p;			/* MODIS channel 6 or 7 uncertainties */
extern uint8	*snow_p, *snowQA_p;		/* SnowCover & snowQA output */

extern uint8	*LandWater_p;			/* Land/water mask */
extern struct COUNTS	cnt;
extern char	RCSidentString[];

extern int32    snow_rejected_warm, snow_rejected_scanangle; /* counters */

/*====================================================================================*/

void ComputeSnow()
{
  char *rcs="$Id: MOD_PR10_Compute_Snow.c,v 1.1 2005/05/19 17:30:23 powell Exp $";

  int32	j;
  float32	ndsi, ndvi, *Save_refl1_p, *Save_refl2_p, *Save_refl4_p, *Save_SfcTemp_p;
  float32	*Save_refl6or7_p;
  static char	FirstTimeFlag = XTRUEX;
  uint8		*Save_LandWater_p, *Save_cloud_p;
  static uint8	SnowCover[MAX_SAMPS],SnowQA[MAX_SAMPS];

  /*** Save pointers for later possible use by Compute_FractionalSnow. ***/
  Save_refl1_p		= refl1_p;
  Save_refl2_p		= refl2_p;
  Save_refl4_p		= refl4_p;
  Save_refl6or7_p	= refl6or7_p;
  Save_cloud_p		= cloud_p;
  Save_SfcTemp_p	= SfcTemp_p;
  Save_LandWater_p	= LandWater_p;

  memset(SnowCover,             L1B_MISSING_DATA,   MAX_SAMPS);
  memset(SnowQA,                QA_OTHER, MAX_SAMPS);

  snow_p   = SnowCover;
  snowQA_p = SnowQA;

  /*** Add the RCS identification to the global string. ***/
  if( FirstTimeFlag == XTRUEX ) {
    FirstTimeFlag = XFALSEX;
    strcat( RCSidentString, "\t" );
    strcat( RCSidentString, rcs );
    strcat( RCSidentString, "\n" );
  }


  /*** Check L1B_FILL value. If the first pixel of a scan is this value, then
       the entire scan is the same.  Avoid unnecessary processing for this scan. ***/
  if( (*refl1_p == L1B_FILL) || (*refl2_p == L1B_FILL) ||
      (*refl4_p == L1B_FILL) || (*refl6or7_p == L1B_FILL) ){
    /*** SnowCover already defaults to L1B_MISSING_DATA. ***/
    memset(SnowQA, QA_OTHER, MAX_SAMPS);
    return;	     /*** Exit the routine.  No further processing needed. ***/
  }


  /*** Process each pixel for this scan. ***/
  for( j=0, snow_p=SnowCover, snowQA_p=SnowQA;
       j<NoSamps; j++, refl1_p++, refl2_p++, refl4_p++, refl6or7_p++, cloud_p++,
	 SfcTemp_p++, uncert6or7_p++, LandWater_p++, snow_p++, snowQA_p++ ) {

    char	Found_QA_OTHER_for_land = XFALSEX;
    
    
    *snow_p = L1B_MISSING_DATA;		 /*** Defailt ***/
    *snowQA_p = QA_GOOD;		 /*** Default ***/

    if( NoisyDetectorFlag || DeadDetectorFlag )
      *snowQA_p = QA_OTHER;


    /*** Mark the highly uncertain band 6 samples.  These are identified by the
	 EV-500_RefSB_Uncert_Indexes SDS. ***/
    if( *uncert6or7_p == UNCERTAINTY_VALUE )
      *snowQA_p = QA_OTHER;



    /*** Determine if this pixel is to be considered land or water. ***/
    if( *LandWater_p == DEEP_OCEAN		||
	*LandWater_p == CONTINENTAL_SHELF_OCEAN ||
	*LandWater_p == SHALLOW_OCEAN 	) {
      *snow_p = OCEAN;		/* Come here for OCEAN */
      *snowQA_p = QA_OCEAN;
      continue;			/* It's ocean, go to bottom of loop. */
    }


    /*** Not ocean, must be land or lakes, etc. ***/
    cnt.LandPixels ++;

    /*** Be sure the L1B input values are usable.  Check all four channels.
	 The snow value will indicate a reason for one of the channels but
	 it will not indicate which channel has the problem; it could be
	 more than one channel and they could have different problems. ***/
    if( (*refl1_p > MAX_USABLE_L1B) || (*refl2_p > MAX_USABLE_L1B) ||
	(*refl4_p > MAX_USABLE_L1B) || (*refl6or7_p > MAX_USABLE_L1B) ) {

      /*** The L1B data is unusable.  Set the snow to the first found
	   reason for its being unusable. ***/
      cnt.InvalidInput ++;
      *snowQA_p = QA_OTHER;
      if( Found_QA_OTHER_for_land == XFALSEX ) {
	Found_QA_OTHER_for_land = XTRUEX;
	cnt.QA_Other ++;
      }
      /*** Check for saturated detector. ***/
      if( (*refl1_p == SATURATED_DETECTOR) || (*refl2_p == SATURATED_DETECTOR) ||
	  (*refl4_p == SATURATED_DETECTOR) || (*refl6or7_p == SATURATED_DETECTOR) ) {
	*snow_p = DETECTOR_SATURATED;
	*snowQA_p = QA_OTHER;
	continue;
      }
      /*** Check for L1B missing. ***/
      if( (*refl1_p == L1B_MISSING) || (*refl2_p == L1B_MISSING) ||
	  (*refl4_p == L1B_MISSING) || (*refl6or7_p == L1B_MISSING) ) {
	*snow_p =  L1B_MISSING_DATA;
	*snowQA_p = QA_OTHER;
	cnt.QA_MissingData ++;
	continue;
      }
      /*** It is unusable for some other reason. ***/
      *snow_p = NO_DECISION;
      *snowQA_p = QA_OTHER;
      cnt.NoDecision ++;
      continue;
    }


    /*** Come here if this pixel is LAND, COASTLINES, SHALLOW_INLAND_WATER,
	 EPHEMERAL_WATER, DEEP_INLAND_WATER. ***/


    /*** Don't compute snow unless the cloud mask can be determined. ***/
    if( ! (*cloud_p & CLOUD_MASK_CF) ) {
      *snow_p = NO_DECISION;
      cnt.NoDecision ++;
      *snowQA_p = QA_OTHER;
      if( Found_QA_OTHER_for_land == XFALSEX ) {
	Found_QA_OTHER_for_land = XTRUEX;
	cnt.QA_Other ++;
      }
      continue;
    }

    /*** Come here if all the L1B input channels have valid values. ***/

    /*** Be sure reflectances are within range. ***/
    if( (*refl1_p <0.0) || (*refl1_p>=1.0) || (*refl2_p <0.0) || (*refl2_p>=1.0) ||
	(*refl4_p <0.0) || (*refl4_p>=1.0) || (*refl6or7_p <0.0) || (*refl6or7_p>=1.0)) {
      *snowQA_p = QA_OTHER;
      if( Found_QA_OTHER_for_land == XFALSEX ) {
	Found_QA_OTHER_for_land = XTRUEX;
	cnt.QA_Other ++;
      }
    }


    /*** Catching division by zero.  If the denominator of the NDSI is zero then a
	 value will be assigned to that pixel and the NDSI calculation will be skipped,
	 otherwise the NDSI will be calculated. ***/
    if( (*refl4_p + *refl6or7_p) > FLT_MIN)
      ndsi = (*refl4_p - *refl6or7_p) / (*refl4_p + *refl6or7_p);
    else
      ndsi = 0.0;

    /*** This function accumulates statistics on NDSI values that were calculated
	 outside of the NDSI theoretical range of -1.0 to 1.0.  A significant number
	 (TBD) of pixels with values outside of this range serves as a signal (warning)
	 to the investigator that some investigating needs to be done to determine the
	 cause of such results. ***/
    if( (ndsi>1.0) || (ndsi<-1.0) ) {
      *snowQA_p = QA_OTHER;
      if( Found_QA_OTHER_for_land == XFALSEX ) {
	Found_QA_OTHER_for_land = XTRUEX;
	cnt.QA_Other ++;
      }
    }

    /*** At this point this pixel's land/water mask has one of the following
	 possible values:   LAND, COASTLINES, SHALLOW_INLAND_WATER, EPHEMERAL_WATER,
	 DEEP_INLAND_WATER ***/

    if( *LandWater_p == SHALLOW_INLAND_WATER || *LandWater_p == DEEP_INLAND_WATER ) {

      if( ndsi >= 0.4  &&  *refl2_p >= 0.11  &&	 *refl4_p >= 0.10)
	*snow_p = LAKE_ICE;
      else
	*snow_p = LAKE;

    } else {

      /*** Come here for LAND, COASTLINES, EPHEMERAL_WATER.  Determine if this pixel
	   is SNOW, SNOW_FOREST, or NO_SNOW.  Two snow regions (grouped criteria
	   tests) are used, snow anywhere and snow in forests. ***/

      if( ndsi >= 0.4 ) {
	if( *refl2_p >= 0.11 && *refl4_p >= 0.10) {
	  *snow_p = SNOW;
	} else {
	  *snow_p = NO_SNOW;
	}

      } else {

	/*** NDSI is less than threshold.  Compute NDVI. ***/
	if( (*refl2_p + *refl1_p) > FLT_MIN)
	  ndvi = (*refl2_p - *refl1_p) / (*refl2_p + *refl1_p);
	else
	  ndvi = 0.0;

	if( ndvi <= 0.0 )
	  *snow_p = NO_SNOW;

	else {

	  /*** Do the polygon test. ***/
	  if( ( ndsi*(YY00-YY01) + ndvi*(XX01-XX00) + (XX00*YY01-YY00*XX01) ) > 0  ||
	      ( ndsi*(YY01-YY02) + ndvi*(XX02-XX01) + (XX01*YY02-YY01*XX02) ) > 0  ||
	      ( ndsi*(YY02-YY03) + ndvi*(XX03-XX02) + (XX02*YY03-YY02*XX03) ) > 0  ||
	      ( ndsi*(YY03-YY04) + ndvi*(XX04-XX03) + (XX03*YY04-YY03*XX04) ) > 0  ||
	      ( ndsi*(YY04-YY05) + ndvi*(XX05-XX04) + (XX04*YY05-YY04*XX05) ) > 0  ||
	      ( ndsi*(YY05-YY06) + ndvi*(XX06-XX05) + (XX05*YY06-YY05*XX06) ) > 0  ||
	      ( ndsi*(YY06-YY07) + ndvi*(XX07-XX06) + (XX06*YY07-YY06*XX07) ) > 0  ||
	      ( ndsi*(YY07-YY08) + ndvi*(XX08-XX07) + (XX07*YY08-YY07*XX08) ) > 0  ||
	      ( ndsi*(YY08-YY09) + ndvi*(XX09-XX08) + (XX08*YY09-YY08*XX09) ) > 0  ||
	      ( ndsi*(YY09-YY10) + ndvi*(XX10-XX09) + (XX09*YY10-YY09*XX10) ) > 0  )
	    *snow_p = NO_SNOW;		/* Outside the polygon */
	  else
	    *snow_p = SNOW_FOREST;	/* Inside the polygon */

	}	/*** End of polygon test block.			       ***/
      }		/*** End of NDSI is less than threshold block.	       ***/

      /* Check Surface Temp */

      if( *SfcTemp_p > MAX_SFC_TEMP_FOR_SNOW  &&
	  ( *snow_p == SNOW  ||  *snow_p == SNOW_FOREST) ){
	*snow_p = NO_SNOW;
	cnt.Snow_Rejected_Warm++;
      }

    }		/*** End of LAND, COASTLINES, EPHEMERAL_WATER block.   ***/

    /*** Only daytime samples are to be processed, see if this pixel is night. ***/
    if( ! (*cloud_p & CLOUD_MASK_3_CF) ) {
      *snow_p = NIGHT;
      cnt.NightPixels ++;
      *snowQA_p = QA_OTHER;
      if( Found_QA_OTHER_for_land == XFALSEX ) {
	Found_QA_OTHER_for_land = XTRUEX;
	cnt.QA_Other ++;
      }
      continue;		/* L1B data is not daytime, go to bottom of loop. */
    }

    /*** The snow for this pixel has been determined without regard to cloudiness. ***/



    /*** Come here to overlay the snow with STANDARD CLOUD TEST. ***/
    /*** Check cloud obstruction bits:
	 00 = cloud			reject it
	 01 = 66% probability of clear	use it
	 10 = 95% probability of clear	use it
	 11 = 99% probability of clear	use it	*/
    if( ! ( (*cloud_p & (CLOUD_MASK_1_CF | CLOUD_MASK_2_CF)) ) )
      *snow_p = CLOUD_OBSCURED;

  }		/*** End: for( j=0.... ***/


  snow_p = SnowCover;
  snowQA_p = SnowQA;
  cloud_p = CloudMask;

  refl1_p	= Save_refl1_p;
  refl2_p	= Save_refl2_p;
  refl4_p	= Save_refl4_p;
  refl6or7_p	= Save_refl6or7_p;
  cloud_p	= Save_cloud_p;
  SfcTemp_p	= Save_SfcTemp_p;
  LandWater_p	= Save_LandWater_p;
}
