/* $Header: /home/powell/mod10/TERRA/v5.0.1/RCS/MOD_PR10_Process_Cloud.c,v 1.1 2005/05/19 17:30:23 powell Exp $

***************************************************************************************
!C

!Description:
	This function is responsible for obtaining one MODIS scan of the cloud
	product MOD35_L2.  The 1 km cloud scan is expanded to 500 meter
	resolution.

Program returns:
	No return code if successful.  If a failure, program halts with the reason
	given in a call to PGS_SMF_GenerateStatusReport.

!Input Parameters:
	CLOUD_INPUT	The PCF number for the cloud input file name
	CallType	Operation to be performed (OPEN,READ,CLOSE)

!Output Parameters:
	cloud_p		cloud product scan pointer

!Revision History:

   $Log: MOD_PR10_Process_Cloud.c,v $
   Revision 1.1  2005/05/19 17:30:23  powell
   Initial revision


!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

    Portions developed at the National Center for Supercomputing
    Applications at the Univ. of Illinois at Urbana-Champaign.

   Principal Investigator, Vincent Salomonson
        Vincent.V.Salomonson@NASA.gov
   Associate Investigator, Dorothy Hall
        Dorothy.K.Hall@NASA.gov
   Scientist/Programmer George Riggs
        George.A.Riggs.1@gsfc.nasa.gov
        301-614-5650
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706
   Programmer, Hugh W. Powell
        Hugh.W.Powell.1@gsfc.nasa.gov
        301-286-2310
        NASA/GSFC/Code 614.1
   Programmer, Nicolo E. DiGirolamo
        Nicolo.E.DiGirolamo.1.1@gsfc.nasa.gov
        301-614-5824
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706

!References and Credits:

!Design Notes:
	This is the Collection 5 version of the code.  This function performs three
	separate functions, as indicated by the CallType input parameter.  It OPENs the
	cloud file at the beginning of the program.  It READs the cloud product for each
	MODIS scan and reduces it from 48 bits to 8 bits.  Each READ call processes one
	scan.  It CLOSEs the file at the conclusion of the program.

!END
***************************************************************************************/

#include <stdio.h>
#include "PGS_IO.h"
#include "PGS_MEM.h"
#include "PGS_MET.h"
#include "hdf.h"
#include "mfhdf.h"
#include "HdfEosDef.h"
#include "MOD_PR10.h"
#include "MOD_PR10_ActionMessages.h"

#define MAX_QAS		10		/* Last dimension of Quality_Assurance */
#define USED_QAS	2

/*** Global Variables ***/
char		cloud_FileName[MAX_NC_NAME];

/*** Pointer to cloud mask. ***/
uint8		*cloud_p, CloudMask[MAX_SAMPS];  /* 1:Cloudy   0:Not Cloudy */

extern char	HaltMsg[];

extern struct CHECK	ck;

/*** The column and row dimensions of the L1B earth view input channels. ***/
/*** The present scan (line) number being processed. ***/
extern int32	NoSamps, NoScans, ThisScanNo;
extern char	RCSidentString[];

/*====================================================================================*/

void ProcessCloudFile( enum OpType CallType )
{
  char *rcs="$Id: MOD_PR10_Process_Cloud.c,v 1.1 2005/05/19 17:30:23 powell Exp $";

  static int32	sd_id, sds_index, sds_id, sdsQ_id, rank, data_type, nattrs;

  static int32	dimsizes [MAX_VAR_DIMS], start [3], edge [3];
  static int32	dimsizesQ[MAX_VAR_DIMS], startQ[3], edgeQ[3];

  static uint8	cloudBuff_1km[3][MAX_SAMPS/2];
  static uint8	cloudQA_1km[MAX_SAMPS/2][USED_QAS];

  /*----------------------------------------------------------------------------------*/

  if( CallType == READ ) {

    int32	j;

    /*** Get a 1 km scan for every two 500 meter scans. ***/
    if( (ThisScanNo/2)*2 == ThisScanNo ) {
      start[1] = startQ[0] = (ThisScanNo/2);

      /*** Read in the Cloud_Mask for one scan. ***/
      if( SDreaddata( sds_id, start, NULL, edge, cloudBuff_1km ) == FAIL )
	HALT2(ACT06,"Can't get Cloud_Mask for this scan.");

      /*** Read in the Quality_Assurance for one scan. ***/
      if( SDreaddata( sdsQ_id, startQ, NULL, edgeQ, cloudQA_1km ) == FAIL )
	HALT2(ACT06,"Can't get Quality_Assurance for this scan.");


      for( j=0; j<NoSamps; j+=2 ) {

	CloudMask[j]  = cloudBuff_1km[0][j/2] &
	  (CLOUD_MASK_CF | CLOUD_MASK_1_CF | CLOUD_MASK_2_CF | CLOUD_MASK_3_CF );

	if( (  cloudQA_1km[j/2][0] & (1<<6)) != 0 &&
	    (cloudBuff_1km[1][j/2] & (1<<6)) == 0 )  CloudMask[j] |= CLOUD_MASK_14_CF;

	if( (  cloudQA_1km[j/2][1] & (1<<3)) != 0 &&
	    (cloudBuff_1km[2][j/2] & (1<<3)) == 0 )  CloudMask[j] |= CLOUD_MASK_19_CF;

	if( (  cloudQA_1km[j/2][1] & (1<<4)) != 0 &&
	    (cloudBuff_1km[2][j/2] & (1<<4)) == 0 )  CloudMask[j] |= CLOUD_MASK_20_CF;

	CloudMask[j+1] = CloudMask[j];
      }

    }

    /*** Set global pointer to beginning of cloud scan. ***/
    cloud_p = CloudMask;

    return;
  }

  /*----------------------------------------------------------------------------------*/

  if( CallType == OPEN ) {

    PGSt_integer	version;
    char		cloud_SDS_Name[MAX_NC_NAME];
    void		*local;


    /*** Add the RCS identification to the global string. ***/
    strcat( RCSidentString, "\t" );
    strcat( RCSidentString, rcs );
    strcat( RCSidentString, "\n" );


    /*** Get GRingPoint's and RangeBeginning's.	 These will be compared with those of
	 the other two input files to be sure that all files are the same granule. ***/
    version = 1;
    if(PGS_MET_GetPCAttr(CLOUD_INPUT, version, ARCHIVE, "GRingPointLatitude.1",
			 ck.cld.GRingPointLatitude ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting GRingPointLatitude.1");

    version = 1;
    if(PGS_MET_GetPCAttr(CLOUD_INPUT, version, ARCHIVE, "GRingPointLongitude.1",
			 ck.cld.GRingPointLongitude ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting GRingPointLongitude.1");

    local = ck.cld.RangeBeginningDate;
    ck.cld.RangeBeginningDate[0] = '\0';
    version = 1;
    if(PGS_MET_GetPCAttr(CLOUD_INPUT, version, CORE,"RangeBeginningDate",&local ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting RangeBeginningDate");

    local = ck.cld.RangeBeginningTime;
    ck.cld.RangeBeginningTime[0] = '\0';
    version = 1;
    if(PGS_MET_GetPCAttr(CLOUD_INPUT, version, CORE,"RangeBeginningTime",&local ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting RangeBeginningTime");


    /*** Retrieve cloud file name from PCF file	 ***/
    if(PGS_PC_GetReference(CLOUD_INPUT,&version, cloud_FileName)!=OK)
      HALT2(ACT04,"Error getting cloud input file name from PCF");

    /*** Open the cloud file. ***/
    if( (sd_id = SDstart( cloud_FileName, DFACC_READ )) == FAIL )
      HALT2(ACT04,"Can't open cloud_file");


    /*** Get the Cloud_Mask SDS. ***/
    if( (sds_index = SDnametoindex( sd_id, "Cloud_Mask" )) == FAIL )
      HALT2(ACT09,"Cannot find SD index for Cloud_Mask in the cloud file");

    if( (sds_id = SDselect( sd_id, sds_index ) ) == FAIL )
      HALT2(ACT09,"Cannot get sds_index for Cloud_Mask");

    if(SDgetinfo( sds_id, cloud_SDS_Name, &rank, dimsizes, &data_type, &nattrs)==FAIL)
      HALT2(ACT09,"Cannot get SDgetinfo for Cloud_Mask");

    if( rank != 3 )
      HALT2(ACT06,"Rank of Cloud_Mask is not 3");

    if( data_type != DFNT_INT8 )
      HALT2(ACT06,"Data type of Cloud_Mask is not DFNT_INT8");

    if( dimsizes[2] != MAX_SAMPS/2 )
      HALT2(ACT06,"No. of samples in  L1B swath != with Cloud_Mask SDS.");

    edge[0] = 3;			/*** Read first 3 bytes of the mask. ***/
    edge[1] = 1;			/*** Read one scan line. ***/
    edge[2] = dimsizes[2];		/*** Read all samples in one scan line. ***/
    start[0] = start[2] = 0;



    /*** Get the Quality_Assurance SDS. ***/
    if( (sds_index = SDnametoindex( sd_id, "Quality_Assurance" )) == FAIL )
      HALT2(ACT09,"Cannot find SD index for Quality_Assurance in the cloud file");

    if( (sdsQ_id = SDselect( sd_id, sds_index ) ) == FAIL )
      HALT2(ACT09,"Cannot get sds_index for Quality_Assurance");

    if(SDgetinfo( sdsQ_id, cloud_SDS_Name, &rank, dimsizesQ, &data_type, &nattrs)==FAIL)
      HALT2(ACT09,"Cannot get SDgetinfo for Quality_Assurance");

    if( rank != 3 )
      HALT2(ACT06,"Rank of Quality_Assurance is not 3");

    if( data_type != DFNT_INT8 )
      HALT2(ACT06,"Data type of Quality_Assurance is not DFNT_INT8");

    if( dimsizesQ[1] != MAX_SAMPS/2 )
      HALT2(ACT06,"No. of samples in  L1B swath != with Quality_Assurance SDS.");

    if( dimsizesQ[2] != MAX_QAS )
      HALT2(ACT06,"QA_Dimension is not consistent with code for Quality_Assurance SDS.");


    edgeQ[0] = 1;
    edgeQ[1] = dimsizesQ[1];
    edgeQ[2] = USED_QAS;

    startQ[1] = 0;
    startQ[2] = 1;

    return;
  }

  /*----------------------------------------------------------------------------------*/

  if( CallType == CLOSE ) {

    if( SDendaccess( sds_id ) == FAIL )
      HALT2(ACT09,"Cannot SDendaccess for Cloud_Mask");

    if( SDendaccess( sdsQ_id ) == FAIL )
      HALT2(ACT09,"Cannot SDendaccess for Quality_Assurance");

    if( SDend( sd_id ) == FAIL )
      HALT2(ACT09,"Cannot SDend for cloud_FileName");
  }
}
