/* $Header: /home/powell/mod10/TERRA/v5.0.1/RCS/MOD_PR10_Process_GEO.c,v 1.1 2005/05/19 17:30:23 powell Exp $

***************************************************************************************
!C

!Description:
	This function is responsible for obtaining solar zenith and sensor zenith angles
	for one MODIS scan of the geo file.	 The Latitude and Longitude 5 km data
	sets are also written to the output file.

Program returns:
	No return code if successful.  If a failure, program halts with the reason
	given in a call to PGS_SMF_GenerateStatusReport.

!Input Parameters:
	GEO_INPUT		The PCF number for the GEO input file name
	CallType		Operation to be performed (OPEN,READ,CLOSE)

!Output Parameters:
	LandWater_p		Land/water mask pointer
	SolarZenith_500m	Solar zenith angle
	Latitude		Latitude  array at 5 km resolution
	Longitude		Longitude array at 5 km resolution

!Revision History:

   $Log: MOD_PR10_Process_GEO.c,v $
   Revision 1.1  2005/05/19 17:30:23  powell
   Initial revision


!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

    Portions developed at the National Center for Supercomputing
    Applications at the Univ. of Illinois at Urbana-Champaign.

   Principal Investigator, Vincent Salomonson
        Vincent.V.Salomonson@NASA.gov
   Associate Investigator, Dorothy Hall
        Dorothy.K.Hall@NASA.gov
   Scientist/Programmer George Riggs
        George.A.Riggs.1@gsfc.nasa.gov
        301-614-5650
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706
   Programmer, Hugh W. Powell
        Hugh.W.Powell.1@gsfc.nasa.gov
        301-286-2310
        NASA/GSFC/Code 614.1
   Programmer, Nicolo E. DiGirolamo
        Nicolo.E.DiGirolamo.1.1@gsfc.nasa.gov
        301-614-5824
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706

!References and Credits:

!Design Notes:
	This is the Collection 5 version of the code.  This function performs three
	separate functions, as indicated by the CallType input parameter.  It OPENs the
	geo file at the beginning of the program.  It READs the sensor zenith angles for
	each satellite scan.  It CLOSEs the file at the conclusion of the program.

!END
***************************************************************************************/

#include <stdio.h>
#include <math.h>
#include "PGS_IO.h"
#include "PGS_MEM.h"
#include "PGS_MET.h"
#include "hdf.h"
#include "mfhdf.h"
#include "HdfEosDef.h"
#include "MOD_PR10.h"
#include "MOD_PR10_ActionMessages.h"

#define PI_VALUE	3.141592654
#define DEG2RAD		(PI_VALUE/180.0)

/*** Global Variables ***/
float32		SolarZenith_500m[MAX_SAMPS];
uint8		*LandWater_p;			/* Pointer */
char		geo_FileName[MAX_NC_NAME];	/* for geo file name */

/*** The column and row dimensions of the L1B earth view input channels. ***/
/*** The present scan (line) number being processed. ***/
extern int32	NoSamps, No5kmSamps, NoScans, No5kmScans, ThisScanNo;
extern char	RCSidentString[], HaltMsg[];

extern struct CHECK	ck;
extern int32		SnowHDFhandle;				/* SD handle	 */
extern int32		SWfidout, SWidSnow;			/* Swath handles */

/*====================================================================================*/

void ProcessGeoFile( enum OpType CallType )
{
  char *rcs="$Id: MOD_PR10_Process_GEO.c,v 1.1 2005/05/19 17:30:23 powell Exp $";

  static float64	SolarZenithScaleFactor_radians;
  static int32		sds_id_SolarZenith, sds_id_SensorZenith, sds_id_LandWater;
  static int32		sd_id, sds_index,dimsizes[MAX_VAR_DIMS], start[2], edge[2];
  static int32		ThisScanNo_1km, ScanNowInBuffer = -999;
  static int16		SolarZenith_1km[MAX_SAMPS/2];
  static uint8		LandWater_1km[MAX_SAMPS/2],	LandWater_500m[MAX_SAMPS];

  /*----------------------------------------------------------------------------------*/

  if( CallType == READ ) {

    int32	j, k;
    float64	coss;

    ThisScanNo_1km = ThisScanNo / 2;

    if( ThisScanNo_1km != ScanNowInBuffer ) {
      start[0] = ThisScanNo_1km;

      /*** Read in the next SolarZenith 1 km scan. Use it for two 500 meter scans.
	   Convert to reciprocal of cosine. ***/
      if( SDreaddata( sds_id_SolarZenith, start, NULL, edge, SolarZenith_1km) == FAIL)
	HALT2(ACT06,"Can't get SolarZenith data for this swath.");
      for( j=k=0; j<NoSamps; j+=2, k++ ) {
	coss = cos( (float64)SolarZenith_1km[k] * SolarZenithScaleFactor_radians );
	if( coss != 0.0 )
	  coss = 1.0/coss;
	else
	  HALT2(ACT06,"Invalid solar zenith angle.");

	SolarZenith_500m[j] = SolarZenith_500m[j+1] = coss;
      }

      /*** Read in the next LandWater 1 km scan. Use it for two 500 meter scans. ***/
      if( SDreaddata( sds_id_LandWater, start, NULL, edge, LandWater_1km) == FAIL )
	HALT2(ACT06,"Can't get land/water data for this swath.");
      for( j=k=0; j<NoSamps; j+=2, k++ )
	LandWater_500m[j] = LandWater_500m[j+1] = LandWater_1km[k];

      ScanNowInBuffer = ThisScanNo_1km;
    }

    /*** Set global pointer to beginning of geo swath. ***/
    LandWater_p	   = LandWater_500m;

    return;
  }

  /*----------------------------------------------------------------------------------*/

  if( CallType == OPEN ) {

    PGSt_integer	version;
    char		geo_SDS_Name[MAX_NC_NAME], attr_name[64];
    int32		attr_index, data_type, count, rank, nattrs;
    float64		SolarZenithScaleFactor;
    void		*local;

    /*** Add the RCS identification to the global string. ***/
    strcat( RCSidentString, "\t" );
    strcat( RCSidentString, rcs );
    strcat( RCSidentString, "\n" );


    /*** Get GRingPoint's and RangeBeginning's.	 These will be compared with those of
	 the other two input files to be sure that all files are the same granule. ***/
    version = 1;
    if(PGS_MET_GetPCAttr(GEO_INPUT, version, CORE, "GRingPointLatitude.1",
			 ck.geo.GRingPointLatitude ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting GRingPointLatitude.1");

    version = 1;
    if(PGS_MET_GetPCAttr(GEO_INPUT, version, CORE, "GRingPointLongitude.1",
			 ck.geo.GRingPointLongitude ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting GRingPointLongitude.1");

    local = ck.geo.RangeBeginningDate;
    ck.geo.RangeBeginningDate[0] = '\0';
    version = 1;
    if(PGS_MET_GetPCAttr(GEO_INPUT, version, CORE, "RangeBeginningDate", &local ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting RangeBeginningDate");

    local = ck.geo.RangeBeginningTime;
    ck.geo.RangeBeginningTime[0] = '\0';
    version = 1;
    if(PGS_MET_GetPCAttr(GEO_INPUT, version, CORE, "RangeBeginningTime", &local ) != OK )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting RangeBeginningTime");


    /*** Retrieve geo file name from PCF file  ***/
    if(PGS_PC_GetReference(GEO_INPUT,&version, geo_FileName) != OK)
      HALT2(ACT04,"Error getting geo input file name from PCF");

    /*** Open the file. ***/
    if( (sd_id = SDstart( geo_FileName, DFACC_READ )) == FAIL )
      HALT2(ACT04,"Can't open geo_file");

    {
      /*** Copy the latitudes and longitudes to the snow output file. ***/
      int32  sds_id_LatLon, numBytes;
      int32	start[] = {2,2}, stride[] = {5,5}, edge[2];
      float32	*float32Pntr;

      if( (SWidSnow=SWattach( SWfidout, "MOD_Swath_Snow" )) == FAIL )
	HALT2(ACT10,"SWattach failed to re-attach swath after calling SWdefgeofield");

      if( (sds_index = SDnametoindex( sd_id, "Latitude" )) == FAIL )
	HALT2(ACT09,"Cannot find SD index for latitude in the geo file");

      if( (sds_id_LatLon = SDselect( sd_id, sds_index ) ) == FAIL )
	HALT2(ACT09,"Cannot get sds_index for latitude");

      if(SDgetinfo(sds_id_LatLon,geo_SDS_Name,&rank,dimsizes, &data_type, &nattrs)==FAIL)
	HALT2(ACT09,"Cannot get SDgetinfo for latitude in geo_FileName");

      if( data_type != DFNT_FLOAT32 )
	HALT2(ACT06,"Data type of Latitude is not DFNT_FLOAT32");

      if( dimsizes[1] != MAX_SAMPS/2 )
	HALT2(ACT06,"No. of samples in GEO latitude inconsistent with compiled value.");


      if( SWdefdim( SWidSnow, "Coarse_swath_lines_5km", No5kmScans ) == FAIL )
	HALT2(ACT10,"SWdefdim failed 'Coarse_swath_lines_5km' for snow cover swath");

      if( SWdefdim( SWidSnow, "Coarse_swath_pixels_5km", No5kmSamps ) == FAIL )
	HALT2(ACT10,"SWdefdim failed 'Coarse_swath_pixels_5km' for snow cover swath");


      if( SWdefgeofield(SWidSnow, "Latitude",
			"Coarse_swath_lines_5km,Coarse_swath_pixels_5km",
			DFNT_FLOAT32, HDFE_NOMERGE ) == FAIL )
	HALT2(ACT10,"SWdefgeofield failed for Latitude");

      if( SWdefgeofield( SWidSnow, "Longitude",
			 "Coarse_swath_lines_5km,Coarse_swath_pixels_5km",
			 DFNT_FLOAT32, HDFE_NOMERGE ) == FAIL )
	HALT2(ACT10,"SWdefgeofield failed for Longitude");



      if( SWdefdimmap( SWidSnow, "Coarse_swath_pixels_5km",
		       "Cross_swath_pixels_500m", 5, 10) == FAIL )
	HALT2(ACT10,"SWdefdimmap failed for Coarse_swath_pixels_5km");

      if( SWdefdimmap( SWidSnow, "Coarse_swath_lines_5km",
		       "Along_swath_lines_500m", 5, 10) == FAIL )
	HALT2(ACT10,"SWdefdimmap failed for Coarse_swath_lines_5km");


      /*** Allocate temporary space for the latitude/longitude arrays. ***/
      numBytes = No5kmScans * No5kmSamps * sizeof(float32);

      float32Pntr = (float32 *)NULL;
      if( PGS_MEM_Malloc( (void **)&float32Pntr, numBytes ) != OK )
	HALT2(ACT10,"Cannot allocate memory for latitude array.");

      edge[0] = No5kmScans;
      edge[1] = No5kmSamps;

      /*** Read in the latitude array. ***/
      if( SDreaddata( sds_id_LatLon, start, stride, edge, float32Pntr) == FAIL)
	HALT2(ACT06,"Can't get the latitude array.");

      if( SWwritefield( SWidSnow, "Latitude", NULL, NULL, NULL, float32Pntr ) == FAIL )
	HALT2(ACT06,"SWwritefield failed for latitude.");

      /*** Read in the longitude array. Assume it has same dimensions as latitude. ***/
      if( (sds_index = SDnametoindex( sd_id, "Longitude" )) == FAIL )
	HALT2(ACT09,"Cannot find SD index for longitude in the geo file");

      if( (sds_id_LatLon = SDselect( sd_id, sds_index ) ) == FAIL )
	HALT2(ACT09,"Cannot get sds_index for longitude");

      if( SDreaddata( sds_id_LatLon, start, stride, edge, float32Pntr) == FAIL)
	HALT2(ACT06,"Can't get the longitude array.");

      if( SWwritefield( SWidSnow, "Longitude", NULL, NULL, NULL, float32Pntr ) == FAIL )
	HALT2(ACT06,"SWwritefield failed for longitude.");

      /*** Free the memory that was used for the LatLon arrays. ***/
      PGS_MEM_Free( float32Pntr );

      /*** Add the product specific attributes to the Latitude and Longitude
	   SDS's in the output file. ***/
      {
	char	*longnameLat = "Coarse 5 km resolution latitude";
	char	*longnameLon = "Coarse 5 km resolution longitude";
	char	*units = "degrees";
	float32	RangeValsLat[] = { -90., 90. };
	float32	RangeValsLon[] = { -180., 180. };
	float32	FillValue = -999.0;
	char	*source = "MOD03 geolocation product; data read from center pixel in 5 km box";

	/*** Set Latitude SDS attributes. ***/
	if( (sds_index = SDnametoindex( SnowHDFhandle, "Latitude" )) == FAIL )
	  HALT2(ACT09,"Cannot find SD index for latitude in the output file");

	if( (sds_id_LatLon = SDselect( SnowHDFhandle, sds_index ) ) == FAIL )
	  HALT2(ACT09,"Cannot get sds_index for latitude in the output file");

	if(SDsetdatastrs(sds_id_LatLon, longnameLat, units, NULL, NULL) == FAIL)
	  HALT2(ACT09,"SDsetdatastrs failed for latitude in output file.");

	if( SDsetattr(sds_id_LatLon,"valid_range",DFNT_FLOAT32,2,RangeValsLat ) == FAIL )
	  HALT2(ACT09,"SDsetattr failure for valid_range for SnowCover");

	if( SDsetattr(sds_id_LatLon,"_FillValue", DFNT_FLOAT32, 1, &FillValue ) == FAIL )
	  HALT2(ACT09,"SDsetattr failure for _FillValue for SnowCover");

	if( SDsetattr(sds_id_LatLon,"source",DFNT_CHAR8,strlen(source),source ) == FAIL )
	  HALT2(ACT09,"SDsetattr failure for source for SnowCover");

	if( SDendaccess( sds_id_LatLon ) == FAIL )
	  HALT2(ACT09,"Cannot SDendaccess for Latitude for the output file.");


	/*** Set Longitude SDS attributes. ***/
	if( (sds_index = SDnametoindex( SnowHDFhandle, "Longitude" )) == FAIL )
	  HALT2(ACT09,"Cannot find SD index for longitude in the output file");

	if( (sds_id_LatLon = SDselect( SnowHDFhandle, sds_index ) ) == FAIL )
	  HALT2(ACT09,"Cannot get sds_index for longitude in the output file");

	if(SDsetdatastrs(sds_id_LatLon, longnameLon, units, NULL, NULL) == FAIL)
	  HALT2(ACT09,"SDsetdatastrs failed for longitude in output file.");

	if( SDsetattr(sds_id_LatLon,"valid_range",DFNT_FLOAT32,2,RangeValsLon) == FAIL )
	  HALT2(ACT09,"SDsetattr failure for valid_range for SnowCover");

	if( SDsetattr(sds_id_LatLon,"_FillValue", DFNT_FLOAT32, 1, &FillValue ) == FAIL )
	  HALT2(ACT09,"SDsetattr failure for _FillValue for SnowCover");

	if( SDsetattr(sds_id_LatLon,"source",DFNT_CHAR8,strlen(source),source ) == FAIL )
	  HALT2(ACT09,"SDsetattr failure for source for SnowCover");

	if( SDendaccess( sds_id_LatLon ) == FAIL )
	  HALT2(ACT09,"Cannot SDendaccess for Longitude for the output file.");
      }
    }

    /*** Select the  SolarZenith SDS. ***/

    if( (sds_index = SDnametoindex( sd_id, "SolarZenith" )) == FAIL )
      HALT2(ACT09,"Cannot find SD index for SolarZenith in the geo file");

    if( (sds_id_SolarZenith = SDselect( sd_id, sds_index ) ) == FAIL )
      HALT2(ACT09,"Cannot get sds_index for SolarZenith");

    if(SDgetinfo( sds_id_SolarZenith, geo_SDS_Name,
		  &rank, dimsizes, &data_type, &nattrs)==FAIL)
      HALT2(ACT09,"Cannot get SDgetinfo for geo_FileName");

    if( rank != 2 )
      HALT2(ACT06,"Rank of SolarZenith is not 2");

    if( data_type != DFNT_INT16 )
      HALT2(ACT06,"Data type of SolarZenith is not DFNT_INT16");

    if( dimsizes[1] != NoSamps/2 )
      HALT2(ACT06,"No. of samples in  L1B swath inconsistent with SolarZenith SDS.");


    if( (attr_index = SDfindattr( sds_id_SolarZenith, "scale_factor" )) == FAIL)
      HALT2(ACT09,"Can't get attr_index for scale_factor for SolarZenith");

    if( SDattrinfo(sds_id_SolarZenith, attr_index, attr_name, &data_type, &count)== FAIL)
      HALT2(ACT09,"Can't get attrinfo for scale_factor for SolarZenith");

    if( (data_type != DFNT_FLOAT64) || (count != 1) )
      HALT2(ACT06,"sds_id_SolarZenith scale_factor has bad data_type and/or count.");

    if( SDreadattr( sds_id_SolarZenith, attr_index, &SolarZenithScaleFactor ) == FAIL )
      HALT2(ACT09,"Can't read attribute for SolarZenith scale_factor in L1B file");

    /*** Factor in degree-to-radian to speed up processing later. ***/
    SolarZenithScaleFactor_radians = DEG2RAD * SolarZenithScaleFactor;


    /*** Select the  SensorZenith SDS. ***/

    if( (sds_index = SDnametoindex( sd_id, "SensorZenith" )) == FAIL )
      HALT2(ACT09,"Cannot find SD index for SensorZenith in the geo file");

    if( (sds_id_SensorZenith = SDselect( sd_id, sds_index ) ) == FAIL )
      HALT2(ACT09,"Cannot get sds_index for SensorZenith");

    if(SDgetinfo( sds_id_SensorZenith, geo_SDS_Name,
		  &rank, dimsizes, &data_type, &nattrs)==FAIL)
      HALT2(ACT09,"Cannot get SDgetinfo for geo_FileName");

    if( rank != 2 )
      HALT2(ACT06,"Rank of SensorZenith is not if");

    if( data_type != DFNT_INT16 )
      HALT2(ACT06,"Data type of SensorZenith is not DFNT_INT16");

    if( dimsizes[1] != NoSamps/2 )
      HALT2(ACT06,"No. of samples in  L1B swath inconsistent with SensorZenith SDS.");


    if( (attr_index = SDfindattr( sds_id_SensorZenith, "scale_factor" )) == FAIL)
      HALT2(ACT09,"Can't get attr_index for scale_factor for SensorZenith");

    if( SDattrinfo(sds_id_SensorZenith, attr_index, attr_name, &data_type, &count)==FAIL)
      HALT2(ACT09,"Can't get attrinfo for scale_factor for SensorZenith");

    if( (data_type != DFNT_FLOAT64) || (count != 1) )
      HALT2(ACT06,"sds_id_SensorZenith scale_factor has bad data_type and/or count.");


    /*** Select the Land/SeaMask SDS. ***/

    if( (sds_index = SDnametoindex( sd_id, "Land/SeaMask" )) == FAIL )
      HALT2(ACT09,"Cannot find SD index for LandWater in the geo file");

    if( (sds_id_LandWater = SDselect( sd_id, sds_index ) ) == FAIL )
      HALT2(ACT09,"Cannot get sds_index for LandWater");

    if(SDgetinfo(sds_id_LandWater,geo_SDS_Name,&rank,dimsizes,&data_type,&nattrs)==FAIL)
      HALT2(ACT09,"Cannot get SDgetinfo for geo_FileName");

    if( rank != 2 )
      HALT2(ACT06,"Rank of LandWater is not 2");

    if( data_type != DFNT_UINT8 )
      HALT2(ACT06,"Data type of LandWater is not DFNT_UINT8");

    if( dimsizes[1] != NoSamps/2 )
      HALT2(ACT06,"No. of samples in  L1B swath inconsistent with LandWater SDS.");


    /*** Use these during subsequent reads. ***/
    edge[0] = 1;
    start[1] = 0;
    edge[1] = dimsizes[1];

    return;
  }

  /*----------------------------------------------------------------------------------*/

  if( CallType == CLOSE ) {

    if( SDendaccess( sds_id_SolarZenith ) == FAIL )
      HALT2(ACT09,"Cannot SDendaccess for SolarZenith");

    if( SDendaccess( sds_id_SensorZenith ) == FAIL )
      HALT2(ACT09,"Cannot SDendaccess for SensorZenith");

    if( SDendaccess( sds_id_LandWater ) == FAIL )
      HALT2(ACT09,"Cannot SDendaccess for LandWater");

    if( SDend( sd_id ) == FAIL )
      HALT2(ACT09,"Cannot SDend for geo_FileName");
  }
}
