/* $Header: /home/powell/mod10/TERRA/v5.0.1/RCS/MOD_PR10_Process_L1B.c,v 1.2 2005/05/19 18:18:15 powell Exp $

***************************************************************************************
!C

!Description:
	This function is responsible for obtaining one MODIS scan of the MODIS
	reflectance data from the L1B file.

Program returns:
	No return code if successful.  If a failure, program halts with the reason
	given in a call to PGS_SMF_GenerateStatusReport.

!Input Parameters:
	L1B_INPUT	The PCF number for the L1B input file name
	CallType	Operation to be performed (OPEN,READ,CLOSE)

!Output Parameters:
	refl1_p		MODIS channel 1 scan pointer
	refl2_p		MODIS channel 2 scan pointer
	refl4_p		MODIS channel 4 scan pointer
	refl6or7_p	MODIS channel 6 scan pointer
	uncert6or7_p	MODIS channel 6 uncertainties pointer

!Revision History:

   $Log: MOD_PR10_Process_L1B.c,v $
   Revision 1.2  2005/05/19 18:18:15  powell
   Fix phone number.

   Revision 1.1  2005/05/19 17:30:23  powell
   Initial revision


!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

    Portions developed at the National Center for Supercomputing
    Applications at the Univ. of Illinois at Urbana-Champaign.

   Principal Investigator, Vincent Salomonson
        Vincent.V.Salomonson@NASA.gov
   Associate Investigator, Dorothy Hall
        Dorothy.K.Hall@NASA.gov
   Scientist/Programmer George Riggs
        George.A.Riggs.1@gsfc.nasa.gov
        301-614-5650
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706
   Programmer, Hugh W. Powell
        Hugh.W.Powell.1@gsfc.nasa.gov
        301-286-2310
        NASA/GSFC/Code 614.1
   Programmer, Nicolo E. DiGirolamo
        Nicolo.E.DiGirolamo.1.1@gsfc.nasa.gov
        301-614-5824
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706

!References and Credits:
	
!Design Notes:
	This is the Collection 5 version of the code.  This function performs three
	separate functions, as indicated by the CallType input parameter.  It OPENs the
	L1B file at the beginning of the program.  It READs the three MODIS channels,
	applies the scale factor and offset, and converts from integer to floating
	point. Each READ call processes one scan.  It CLOSEs the file at the conclusion
	of the program.

	The valid L1B values are actually "reflectance * cos( solar zenith)".  This
	routine multiplies these by "1.0/cos(solar zenith)" to get the reflectance.  For
	the invalid L1B values (those > 32767) the original L1B are unchanged except for
	conversion from uint16 to float32.

!END
***************************************************************************************/

#include <stdio.h>
#include "PGS_IO.h"
#include "PGS_MEM.h"
#include "PGS_MET.h"
#include "hdf.h"
#include "mfhdf.h"
#include "HdfEosDef.h"
#include "MOD_PR10.h"
#include "MOD_PR10_ActionMessages.h"

#define MODIS_CHAN1_INDEX	0
#define MODIS_CHAN2_INDEX	1
#define MODIS_CHAN4_INDEX	1
#define MODIS_TERRA_INDEX	3

/*** Global Variables ***/
float32		ValidEVobservations[38], SaturatedEVobservations[38];
int8		DeadDetectorList[490], NoisyDetectorList[490];
extern struct COUNTS	cnt;


/*** Pointers to the 3 MODIS reflectances, after scaling and offsets. ***/
float32		*refl1_p, *refl2_p, *refl4_p, *refl6or7_p;
uint8		*uncert6or7_p;

extern struct CHECK	ck;

extern char	HaltMsg[];
extern char	in_FileName[];
extern int32	NoScans, DayModeScans;
extern float32	SolarZenith_500m[MAX_SAMPS];

/*** The column and row dimensions of the L1B earth view input channels. ***/
/*** The present scan (line) number being processed. ***/
extern int32	NoSamps, No5kmSamps, NoScans, No5kmScans, ThisScanNo;
extern char	RCSidentString[];

extern struct Meta5km	km;

/*====================================================================================*/

void ProcessL1Bfile( enum OpType CallType )
{
  char *rcs="$Id: MOD_PR10_Process_L1B.c,v 1.2 2005/05/19 18:18:15 powell Exp $";

  static int32		sd_id, sds_index250, sds_index500;
  static int32		SWfidin, SWidin;		/* Swath handles */

  static int16		MODIS_Chan6or7_index;		/* Set by OPEN */

  /*** Scales and offsets used for converting the MODIS earth view channels. ***/
  static float32	 scaleBand1,  scaleBand2,  scaleBand4,  scaleBand6or7;
  static float32	offsetBand1, offsetBand2, offsetBand4, offsetBand6or7;

  /*----------------------------------------------------------------------------------*/

  if( CallType == READ ) {

    int16  j;

    static uint16	EV250_ch1[MAX_SAMPS];
    static float32	refl_chan1[MAX_SAMPS];

    static uint16	EV250_ch2[MAX_SAMPS];
    static float32	refl_chan2[MAX_SAMPS];

    static uint16	EV500_ch4[MAX_SAMPS];
    static float32	refl_chan4[MAX_SAMPS];

    static uint16	EV500_ch6or7[MAX_SAMPS];
    static float32	refl_chan6or7[MAX_SAMPS];
    static uint8	uncert_chan6or7[MAX_SAMPS];

    static int32	start[3] = {9999,9999,0},  edge[3]  = {1,1,9999};


    edge[2] = NoSamps;		/*** Number of samples in this scan. ***/
    start[1] = ThisScanNo;	/*** Which scan line to read ***/



    /*** Read in MODIS channel 1.  250 meters aggregated to 500 meters. ***/
    start[0] = MODIS_CHAN1_INDEX;
    if(SWreadfield( SWidin,"EV_250_Aggr500_RefSB", start, NULL, edge, EV250_ch1) == FAIL)
      HALT2(ACT06,"Failure reading EV-250_Aggr500_RefSB MODIS channel 1");
    for(j=0; j<NoSamps; j++) {
      if( EV250_ch1[j] <= MAX_USABLE_L1B )
	refl_chan1[j] =	 /* Get reflectance by multiplying by 1/cos(SolZenith). */
	  (((float32)EV250_ch1[j]-offsetBand1) * scaleBand1) * SolarZenith_500m[j];
      else
	refl_chan1[j] = EV250_ch1[j];		/* Invalid value */
    }


    /*** Read in MODIS channel 2.  250 meters aggregated to 500 meters. ***/
    start[0] = MODIS_CHAN2_INDEX;
    if(SWreadfield( SWidin,"EV_250_Aggr500_RefSB", start, NULL, edge, EV250_ch2) == FAIL)
      HALT2(ACT06,"Failure reading EV-250_Aggr500_RefSB MODIS channel 2");
    for(j=0; j<NoSamps; j++) {
      if( EV250_ch2[j] <= MAX_USABLE_L1B )
	refl_chan2[j] =	 /* Get reflectance by multiplying by 1/cos(SolZenith). */
	  (((float32)EV250_ch2[j]-offsetBand2) * scaleBand2) * SolarZenith_500m[j];
      else
	refl_chan2[j] = EV250_ch2[j];		/* Invalid value */
    }


    /*** Read in MODIS channel 4.  500 meters ***/
    start[0] = MODIS_CHAN4_INDEX;
    if(SWreadfield( SWidin, "EV_500_RefSB", start, NULL, edge, EV500_ch4) == FAIL)
      HALT2(ACT06,"Failure reading EV_500 MODIS channel 4");
    for(j=0; j<NoSamps; j++) {
      if( EV500_ch4[j] <= MAX_USABLE_L1B )
	refl_chan4[j] =	 /* Get reflectance by multiplying by 1/cos(SolZenith). */
	  (((float32)EV500_ch4[j]-offsetBand4) * scaleBand4) * SolarZenith_500m[j];
      else
	refl_chan4[j] = EV500_ch4[j];		/* Invalid value */
    }


    /*** Read in MODIS channel 6 or 7.  500 meters ***/
    start[0] = MODIS_Chan6or7_index;
    if(SWreadfield( SWidin, "EV_500_RefSB", start, NULL, edge, EV500_ch6or7) == FAIL)
      HALT2(ACT06,"Failure reading EV_500 MODIS channel 6 or 7");
    for(j=0; j<NoSamps; j++) {
      if( EV500_ch6or7[j] <= MAX_USABLE_L1B )
	refl_chan6or7[j] =    /* Get reflectance by multiplying by 1/cos(SolZenith). */
	  (((float32)EV500_ch6or7[j]-offsetBand6or7)*scaleBand6or7)*SolarZenith_500m[j];
      else
	refl_chan6or7[j] = EV500_ch6or7[j];		/* Invalid value */
    }
    /*** Read in MODIS channel 6 or 7 uncertainties. ***/
    start[0] = MODIS_Chan6or7_index;
    if(SWreadfield( SWidin, "EV_500_RefSB_Uncert_Indexes", start,
		    NULL, edge, uncert_chan6or7) == FAIL)
      HALT2(ACT06,"Failure reading EV_500 MODIS channel 6 or 7 uncertainties");


    /*** Set global pointers to the three MODIS scans. ***/
    refl1_p   = refl_chan1;
    refl2_p   = refl_chan2;
    refl4_p   = refl_chan4;
    refl6or7_p   = refl_chan6or7;
    uncert6or7_p = uncert_chan6or7;

    return;
  }

  /*----------------------------------------------------------------------------------*/

  if( CallType == OPEN ) {

    int32		attr_index, data_type, count, NightModeScans;
    PGSt_integer	version;
    char		attr_name[64];
    void		*local;


    /*** Add the RCS identification to the global string. ***/
    strcat( RCSidentString, "\t" );
    strcat( RCSidentString, rcs );
    strcat( RCSidentString, "\n" );

    /*** Use channel 6 for TERRA. ***/
    MODIS_Chan6or7_index = MODIS_TERRA_INDEX;


    /*** Get GRingPoint's and RangeBeginning's.	 These will be compared with those of
	 the other two input files to be sure that all files are the same granule. ***/
    version = 1;
    if(PGS_MET_GetPCAttr(L1B_INPUT, version, CORE, "GRingPointLatitude.1",
			 ck.l1b.GRingPointLatitude ) != PGS_S_SUCCESS )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting GRingPointLatitude.1");

    version = 1;
    if(PGS_MET_GetPCAttr(L1B_INPUT, version, CORE, "GRingPointLongitude.1",
			 ck.l1b.GRingPointLongitude ) != PGS_S_SUCCESS )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting GRingPointLongitude.1");

    local = ck.l1b.RangeBeginningDate;
    ck.l1b.RangeBeginningDate[0] = '\0';
    version = 1;
    if(PGS_MET_GetPCAttr(L1B_INPUT, version, CORE, "RangeBeginningDate",
			 &local ) != PGS_S_SUCCESS )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting RangeBeginningDate");

    local = ck.l1b.RangeBeginningTime;
    ck.l1b.RangeBeginningTime[0] = '\0';
    version = 1;
    if(PGS_MET_GetPCAttr(L1B_INPUT, version, CORE, "RangeBeginningTime",
			 &local ) != PGS_S_SUCCESS )
      HALT2(ACT07,"PGS_MET_GetPCAttr failure getting RangeBeginningTime");


    /*** Open the L1B input file. ***/
    if( (sd_id = SDstart( in_FileName, DFACC_READ )) == FAIL )
      HALT2(ACT04,"Can't open the L1B input file.");


    /*** Get "Dead Detector List."  Values will be needed for all bands. ***/
    if( (attr_index = SDfindattr( sd_id, "Dead Detector List" )) == FAIL)
      HALT2(ACT09,"Can't get attr_index for 'Dead Detector List' in L1B file");

    if( SDattrinfo( sd_id, attr_index, attr_name, &data_type, &count ) == FAIL )
      HALT2(ACT09,"Can't get attrinfo for 'Dead Detector List' in L1B file");

    if( data_type != DFNT_INT8  || count != sizeof(DeadDetectorList) )
      HALT2(ACT09,"Dead Detector List is wrong type or wrong size");

    if( SDreadattr( sd_id, attr_index, DeadDetectorList ) == FAIL )
      HALT2(ACT09,"Can't read attribute for DeadDetectorList in L1B file");


    /*** Get "Noisy Detector List."  Values will be needed for all bands. ***/
    if( (attr_index = SDfindattr( sd_id, "Noisy Detector List" )) == FAIL)
      HALT2(ACT09,"Can't get attr_index for 'Noisy Detector List' in L1B file");

    if( SDattrinfo( sd_id, attr_index, attr_name, &data_type, &count ) == FAIL )
      HALT2(ACT09,"Can't get attrinfo for 'Noisy Detector List' in L1B file");

    if( data_type != DFNT_INT8  || count != sizeof(NoisyDetectorList) )
      HALT2(ACT09,"Noisy Detector List is wrong type or wrong size");

    if( SDreadattr( sd_id, attr_index, NoisyDetectorList ) == FAIL )
      HALT2(ACT09,"Can't read attribute for NoisyDetectorList in L1B file");


    /*** Get "%Valid EV Observations" for MODIS bands 1,2,4,6 (index 0,1,3,5).	These
	 will be local attributes for the Snow Cover SDS. ***/
    if( (attr_index = SDfindattr( sd_id, "%Valid EV Observations" )) == FAIL)
      HALT2(ACT09,"Can't get attr_index for '%Valid EV Observations' in L1B file");

    if( SDattrinfo( sd_id, attr_index, attr_name, &data_type, &count ) == FAIL )
      HALT2(ACT09,"Can't get attrinfo for '%Valid EV Observations' in L1B file");

    if( data_type != DFNT_FLOAT32  ||
	count	  != sizeof(ValidEVobservations) / sizeof(float32) )
      HALT2(ACT09,"%ValidEVobservations is wrong type or wrong size");

    if( SDreadattr( sd_id, attr_index, ValidEVobservations ) == FAIL )
      HALT2(ACT09,"Can't read attribute for '&Valid EV Observations' in L1B file");


    /*** Get "%Saturated EV Observations" for MODIS bands 1,2,4,6 (index 0,1,3,5).  These
	 will be local attributes for the Snow Cover SDS. ***/
    if( (attr_index = SDfindattr( sd_id, "%Saturated EV Observations" )) == FAIL)
      HALT2(ACT09,"Can't get attr_index for '%Saturated EV Observations' in L1B file");

    if( SDattrinfo( sd_id, attr_index, attr_name, &data_type, &count ) == FAIL )
      HALT2(ACT09,"Can't get attrinfo for '%Saturated EV Observations' in L1B file");

    if( data_type != DFNT_FLOAT32  ||
	count	  != sizeof(SaturatedEVobservations) / sizeof(float32) )
      HALT2(ACT09,"%SaturatedEVobservations is wrong type or wrong size");

    if( SDreadattr( sd_id, attr_index, SaturatedEVobservations ) == FAIL )
      HALT2(ACT09,"Can't read attribute for '&Saturated EV Observations' in L1B file");


    /*** Get "Number of Day mode scans". ***/
    if( (attr_index = SDfindattr( sd_id, "Number of Day mode scans" )) == FAIL)
      HALT2(ACT09,"Can't get attr_index for 'Number of Day mode scans' in L1B file");

    if( SDattrinfo( sd_id, attr_index, attr_name, &data_type, &count ) == FAIL )
      HALT2(ACT09,"Can't get attrinfo for 'Number of Day mode scans' in L1B file");

    if( SDreadattr( sd_id, attr_index, &DayModeScans ) == FAIL )
      HALT2(ACT09,"Can't read attribute for 'Number of Day mode scans' in L1B file");


    /*** Get "Number of Night mode scans". ***/
    if( (attr_index = SDfindattr( sd_id, "Number of Night mode scans" )) == FAIL)
      HALT2(ACT09,"Can't get attr_index for 'Number of Night mode scans' in L1B file");

    if( SDattrinfo( sd_id, attr_index, attr_name, &data_type, &count ) == FAIL)
      HALT2(ACT09,"Can't get attrinfo for 'Number of Night mode scans' in L1B file");

    if( SDreadattr( sd_id, attr_index, &NightModeScans ) == FAIL )
      HALT2(ACT09,"Can't read attribute for 'Number of Night mode scans' in L1B file");


    /*** Set the DayNightFlag meta data to be written later in MakeMeta. ***/
    if( DayModeScans >0 && NightModeScans >0 )	strcpy( km.DayNightFlag, "Both" );
    if( DayModeScans >0 && NightModeScans==0 )	strcpy( km.DayNightFlag, "Day" );
    if( DayModeScans==0 && NightModeScans >0 )	strcpy( km.DayNightFlag, "Night" );
    if( DayModeScans==0 && NightModeScans==0 )
      HALT2(ACT06,"The L1B input has no day mode scans and no night mode scans.");


    /*** Get swath handle ***/
    if( (SWfidin = SWopen( in_FileName, DFACC_READ )) == FAIL )
      HALT2(ACT04,"SWopen failed for L1B file");

    if( (SWidin = SWattach( SWfidin, "MODIS_SWATH_Type_L1B" )) == FAIL )
      HALT2(ACT09,"SWattach failed for L1B file");

    if( SWdiminfo(SWidin, "Band_250M") != 2 )
      HALT2(ACT06,"Swath dimension for Band_250M != 2");

    if( SWdiminfo(SWidin, "Band_500M") != 5 )
      HALT2(ACT06,"Swath dimension for Band_500M != 5");

    if( (NoScans =  SWdiminfo(SWidin, "20*nscans")) == -1 )
      HALT2(ACT06,"Swath dimension for 20*nscans failed.");

    if( (NoSamps =  SWdiminfo(SWidin, "2*Max_EV_frames")) == -1 )
      HALT2(ACT06,"Swath dimension for 2*Max_EV_frames failed.");

    if( NoSamps > MAX_SAMPS )
      HALT2(ACT06,"2*Max_EV_Frames is greater than MAX_SAMPS");

    cnt.AllPixelsProcessed = NoScans * NoSamps;


    /*** Process 250m reflectance scales and offsets. ***/
    {
      float32	floatBuf[2];
      char	band250name[256];
      static int32   sds_id250,rank250,data_type250,dimsizes250[MAX_VAR_DIMS],nattrs250;

      if( (sds_index250 = SDnametoindex( sd_id, "EV_250_Aggr500_RefSB" )) == FAIL )
	HALT2(ACT09,"sds_index250 failed in L1B file");

      if( (sds_id250 = SDselect( sd_id, sds_index250 ) ) == FAIL )
	HALT2(ACT09,"sds_id250 failed in L1B file");

      if(SDgetinfo( sds_id250, band250name, &rank250,
		    dimsizes250, &data_type250, &nattrs250) == FAIL )
	HALT2(ACT09,"SDgetinfo failed for band250name in L1B file");


      /*** Get 250m scales ***/
      if( (attr_index = SDfindattr( sds_id250, "reflectance_scales" )) == FAIL)
	HALT2(ACT09,"Can't get attr_index for 250 'reflectance_scales' in L1B file");

      if( SDattrinfo( sds_id250, attr_index, attr_name, &data_type, &count ) == FAIL)
	HALT2(ACT09,"Can't get attrinfo for 250 'reflectance_scales' in L1B file");

      if( (data_type != DFNT_FLOAT32) || (count != 2) )
	HALT2(ACT06,"sds_id250 reflectance_scales has bad data_type and/or count.");

      if( SDreadattr( sds_id250, attr_index, floatBuf ) == FAIL )
	HALT2(ACT09,"Can't read attribute for 250 'reflectance_scales' in L1B file");

      scaleBand1 = floatBuf[0];
      scaleBand2 = floatBuf[1];



      /*** Get 250m offsets ***/
      if( (attr_index = SDfindattr( sds_id250, "reflectance_offsets" )) == FAIL)
	HALT2(ACT09,"Can't get attr_index for 250 'reflectance_offsets' in L1B file");

      if( SDattrinfo( sds_id250, attr_index, attr_name, &data_type, &count ) == FAIL)
	HALT2(ACT09,"Can't get attrinfo for 250 'reflectance_offsets' in L1B file");

      if( (data_type != DFNT_FLOAT32) || (count != 2) )
	HALT2(ACT06,"sds_id250 reflectance_offsets has bad data_type and/or count.");

      if( SDreadattr( sds_id250, attr_index, floatBuf ) == FAIL )
	HALT2(ACT09,"Can't read attribute for 250 'reflectance_offsets' in L1B file");

      offsetBand1 = floatBuf[0];
      offsetBand2 = floatBuf[1];

      if( SDendaccess( sds_id250 ) == FAIL )
	HALT2(ACT09,"Cannot SDendaccess for sds_id250");
    }


    /*** Process 500m reflectance scales and offsets. ***/
    {
      char	band500name[256];
      float32	floatBuf[5];
      static int32   sds_id500,rank500,data_type500,dimsizes500[MAX_VAR_DIMS],nattrs500;

      if( (sds_index500 = SDnametoindex( sd_id, "EV_500_RefSB" )) == FAIL )
	HALT2(ACT09,"sds_index500 failed in L1B file");

      if( (sds_id500 = SDselect( sd_id, sds_index500 ) ) == FAIL )
	HALT2(ACT09,"sds_id500 failed in L1B file");

      if(SDgetinfo( sds_id500, band500name, &rank500,
		    dimsizes500, &data_type500, &nattrs500) == FAIL )
	HALT2(ACT09,"SDgetinfo failed for band500name in L1B file");


      /*** Get 500m scales ***/
      if( (attr_index = SDfindattr( sds_id500, "reflectance_scales" )) == FAIL)
	HALT2(ACT09,"Can't get attr_index for 500 'reflectance_scales' in L1B file");

      if( SDattrinfo( sds_id500, attr_index, attr_name, &data_type, &count ) == FAIL)
	HALT2(ACT09,"Can't get attrinfo for 500 'reflectance_scales' in L1B file");

      if( (data_type != DFNT_FLOAT32) || (count != 5) )
	HALT2(ACT06,"sds_id500 reflectance_scales has bad data_type and/or count.");

      if( SDreadattr( sds_id500, attr_index, floatBuf ) == FAIL )
	HALT2(ACT09,"Can't read attribute for 500 'reflectance_scales' in L1B file");

      scaleBand4    = floatBuf[MODIS_CHAN4_INDEX];
      scaleBand6or7 = floatBuf[MODIS_Chan6or7_index];


      /*** Get 500m offsets ***/
      if( (attr_index = SDfindattr( sds_id500, "reflectance_offsets" )) == FAIL)
	HALT2(ACT09,"Can't get attr_index for 500 'reflectance_offsets' in L1B file");

      if( SDattrinfo( sds_id500, attr_index, attr_name, &data_type, &count ) == FAIL)
	HALT2(ACT09,"Can't get attrinfo for 500 'reflectance_offsets' in L1B file");

      if( (data_type != DFNT_FLOAT32) || (count != 5) )
	HALT2(ACT06,"sds_id500 reflectance_offsets has bad data_type and/or count.");

      if( SDreadattr( sds_id500, attr_index, floatBuf ) == FAIL )
	HALT2(ACT09,"Can't read attribute for 500 'reflectance_offsets' in L1B file");

      offsetBand4    = floatBuf[MODIS_CHAN4_INDEX];
      offsetBand6or7 = floatBuf[MODIS_Chan6or7_index];

      if( SDendaccess( sds_id500 ) == FAIL )
	HALT2(ACT10,"Cannot SDendaccess for sds_id500");
    }

    return;
  }

  /*----------------------------------------------------------------------------------*/

  if( CallType == CLOSE ) {
    if( SWdetach( SWidin ) == FAIL )
      HALT2(ACT10,"Failure with SWdetach for L1B");

    if( SWclose( SWfidin ) == FAIL )
      HALT2(ACT10,"Failure with SWclose for L1B");

    if( SDend( sd_id ) == FAIL )
      HALT2(ACT10,"Failure with SDend for sd_id for L1B");
  }
}
