/* $Header: /home/powell/mod10/TERRA/v5.0.1/RCS/MOD_PR10_Process_SnowFile.c,v 1.2 2005/05/19 18:18:59 powell Exp $

***************************************************************************************
!C

!Description:
	This function is responsible for writing out the computed snow cover and snow QA
	SDS's and SDS attributes to the snow output file.

Program returns:
	No return code if successful.  If a failure, program halts with the reason
	given in a call to PGS_SMF_GenerateStatusReport.

!Input Parameters:
	SNOW_OUTPUT	The PCF number for the snow output file name
	CallType	Operation to be performed (OPEN,WRITE,CLOSE)
	snow_p		Snow cover scan pointer
	snowQA_p	Snow QA scan pointer

!Output Parameters:
	None

!Revision History:

   $Log: MOD_PR10_Process_SnowFile.c,v $
   Revision 1.2  2005/05/19 18:18:59  powell
   Fix phone number.

   Revision 1.1  2005/05/19 17:30:23  powell
   Initial revision


!Team-unique Header:
  This software was developed by:

    MODIS Land Science Team for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA <contract or task number>

    MODIS Land Science Team Support Group for the National Aeronautics and
    Space Administration, Goddard Space Flight Center, under
    NASA Task 92-012-05

    MODIS Science Data Support Team for the National Aeronautics and Space
    Administration, Goddard Space Flight Center, under contract NAS5-32373.

    Portions developed at the National Center for Supercomputing
    Applications at the Univ. of Illinois at Urbana-Champaign.

   Principal Investigator, Vincent Salomonson
        Vincent.V.Salomonson@NASA.gov
   Associate Investigator, Dorothy Hall
        Dorothy.K.Hall@NASA.gov
   Scientist/Programmer George Riggs
        George.A.Riggs.1@gsfc.nasa.gov
        301-614-5650
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706
   Programmer, Hugh W. Powell
        Hugh.W.Powell.1@gsfc.nasa.gov
        301-286-2310
        NASA/GSFC/Code 614.1
   Programmer, Nicolo E. DiGirolamo
        Nicolo.E.DiGirolamo.1.1@gsfc.nasa.gov
        301-614-5824
        NASA/GSFC/Code 614.1
        Science Systems & Applications, Inc.
        10210 Greenbelt Road, Suite 600, Lanham, MD 20706

!References and Credits:

!Design Notes:
	This is the Collection 5 version of the code.  This function performs three
	separate functions, as indicated by the CallType input parameter.  It OPENs the
	output file at the beginning of the program.  It WRITEs the snow cover and snow
	QA scans.  Each WRITE call processons one scan.  It CLOSEs the file and write
	out all the attribute data at the end of the program.

!END
***************************************************************************************/

#include <stdio.h>
#include <string.h>
#include "PGS_IO.h"
#include "PGS_MEM.h"
#include "hdf.h"
#include "hntdefs.h"
#include "mfhdf.h"
#include "HdfEosDef.h"
#include "MOD_PR10.h"
#include "MOD_PR10_ActionMessages.h"

#define MAX_ACCEPTABLE	15		/* Used for setting QAoverallFlag */

/*** Prototypes ***/
void UpdateCounts( uint8 *, uint8 * );


/*** Global Variables ***/
char			out_FileName[PGSd_PC_FILE_PATH_MAX];	/* for out file name */
enum AutoQualFlag	AutomaticQualityFlag;
extern struct Meta5km	km;
uint8			*snow_p, *snowQA_p, *FracSnow_p;
float32			pct_nominal, pct_abnormal;
int32			SnowHDFhandle;
int32			SWfidout, SWidSnow;			/* Swath handles */

extern char		HaltMsg[];
extern struct COUNTS	cnt;
extern float32		ValidEVobservations[], SaturatedEVobservations[];

/*** The column and row dimensions of the L1B earth view input channels. ***/
/*** The present scan (line) number being processed. ***/
extern int32	NoSamps, NoScans, ThisScanNo;
extern char	RCSidentString[];

/*====================================================================================*/

void ProcessSnowFile( enum OpType CallType )
{
  char *rcs="$Id: MOD_PR10_Process_SnowFile.c,v 1.2 2005/05/19 18:18:59 powell Exp $";

  /*----------------------------------------------------------------------------------*/

  if( CallType == WRITE ) {

    static int32	start[2] = {9999,0},	edge[2]  = {1,9999};
    edge[1] = NoSamps;


    start[0] = ThisScanNo;
    if( SWwritefield( SWidSnow, "Snow_Cover", start, NULL, edge, snow_p ) == FAIL )
      HALT2(ACT06,"SWwritefield failed for a scan of Snow_Cover");

    if( SWwritefield( SWidSnow,"Snow_Cover_Pixel_QA",start,NULL,edge,snowQA_p) == FAIL )
      HALT2(ACT06,"SWwritefield failed for a scan of SnowQA");

    if( SWwritefield( SWidSnow, "Fractional_Snow_Cover", start,
		      NULL, edge, FracSnow_p ) == FAIL )
      HALT2(ACT06,"SWwritefield failed for a scan of Fractional_Snow_Cover");

    /*** Update the snow and QA counts. ***/
    UpdateCounts( snow_p, snowQA_p );

    return;
  }

  /*----------------------------------------------------------------------------------*/

  if( CallType == OPEN ) {

    int32		hdfID;
    PGSt_integer	version;				/* for HDF input file */


    /*** Add the RCS identification to the global string. ***/
    strcat( RCSidentString, "\t" );
    strcat( RCSidentString, rcs );
    strcat( RCSidentString, "\n" );

    /*** Default values for AutomaticQualityFlag. ***/
    AutomaticQualityFlag = PASSED;
    strcpy( km.AutomaticQualityFlagExplanation, "No automatic quality assessment done in the PGE" );

    version = 1;
    if(PGS_PC_GetReference(SNOW_OUTPUT, &version, out_FileName) != PGS_S_SUCCESS)
      HALT2(ACT04,"Error getting snow output file name from PCF");

    /*** Get swath handle ***/
    if( (SWfidout = SWopen( out_FileName, DFACC_CREATE )) == FAIL )
      HALT2(ACT08,"SWopen failed for snow output file");

    /*** Link the swath handle to to toolkit. ***/
    if( EHidinfo(SWfidout, &hdfID, &SnowHDFhandle) == FAIL )
      HALT2(ACT10,"EHindinfo failure");

    /*** Create the swath ***/
    if( (SWidSnow=SWcreate( SWfidout, "MOD_Swath_Snow" )) == FAIL )
      HALT2(ACT10,"SWcreate failed for snow cover swath");



    /*** Define the swath dimensions. ***/
    if( SWdefdim( SWidSnow, "Along_swath_lines_500m", NoScans ) == FAIL )
      HALT2(ACT10,"SWdefdim failed 'Along track' for snow cover swath");


    if( SWdefdim( SWidSnow, "Cross_swath_pixels_500m", NoSamps ) == FAIL )
      HALT2(ACT10,"SWdefdim failed 'Cross track' for snow cover swath");


    if( SWdefdatafield( SWidSnow, "Snow_Cover",
			"Along_swath_lines_500m,Cross_swath_pixels_500m",
			DFNT_UINT8, HDFE_NOMERGE ) == FAIL )
      HALT2(ACT10,"SWdefdatafield failed Snow_Cover");

    if( SWdefdatafield( SWidSnow, "Snow_Cover_Pixel_QA",
			"Along_swath_lines_500m,Cross_swath_pixels_500m",
			DFNT_UINT8, HDFE_NOMERGE ) == FAIL )
      HALT2(ACT10,"SWdefdatafield failed Snow_Cover_Pixel_QA");

    if( SWdefdatafield( SWidSnow, "Fractional_Snow_Cover",
			"Along_swath_lines_500m,Cross_swath_pixels_500m",
			DFNT_UINT8, HDFE_NOMERGE ) == FAIL )
      HALT2(ACT10,"SWdefdatafield failed Fractional_Snow_Cover");


    if( SWdetach( SWidSnow ) == FAIL )
      HALT2(ACT10,"SWdetach failed after calling SWdefgeofield");

    return;
  }
  /*----------------------------------------------------------------------------------*/

  if( CallType == CLOSE ) {

    int32	sds_id, sds_index;
    char	*longname	= "Snow covered land";
    char	*longnameQA	= "Snow cover per pixel thematic QA";
    char	*longnameFrac	= "Fractional snow cover, 500m";
    char	*units		= "none";
    char	*format		= "I3";
    char	*coordsys	= "cartesian";
    char	*nadirName	= "500 m";
    char	KeyStg[256];
    uint8	RangeVals[2]	= {0,254},	FillValue = FILL_VALUE;
    uint8	RangeValsQA[2]	= {0,254};



    if( SWdetach( SWidSnow ) == FAIL )
      HALT2(ACT10,"SWdetach failed for snow cover swath");

    /*** Write out the "Snow_Cover" SDS attributes. ***/
    if( (sds_index = SDnametoindex( SnowHDFhandle, "Snow_Cover" )) == FAIL )
      HALT2(ACT09,"Cannot find SD index for Snow_Cover in the output file");

    if( (sds_id = SDselect( SnowHDFhandle, sds_index)) == FAIL )
      HALT2(ACT09,"SDselect failure for Snow_Cover");

    if(SDsetdatastrs(sds_id, longname, units, format, coordsys) == FAIL)
      HALT2(ACT09,"SDsetdatastrs failed for Snow_Cover");

    if( SDsetattr(sds_id,"valid_range", DFNT_UINT8, 2, RangeVals ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for valid_range for Snow_Cover");

    if( SDsetattr(sds_id, "_FillValue", DFNT_UINT8, 1, &FillValue ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for _FillValue for Snow_Cover");

      sprintf(KeyStg,"%i=missing data, %i=no decision, %i=night, %i=no snow, %i=lake, %i=ocean, %i=cloud, %i=lake ice, %i=snow, %i=detector saturated, %i=fill",
	      L1B_MISSING_DATA, NO_DECISION, NIGHT, NO_SNOW, LAKE, OCEAN,
	      CLOUD_OBSCURED, LAKE_ICE, SNOW, DETECTOR_SATURATED, FILL_VALUE );

    if( SDsetattr(sds_id, "Key:", DFNT_CHAR8, strlen(KeyStg), KeyStg ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for Key for Snow_Cover");

    if( SDsetattr(sds_id, "Nadir_data_resolution", DFNT_CHAR8,
		  strlen(nadirName), nadirName ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for Nadir_data_resolution for Snow_Cover");

    if( SDsetattr(sds_id, "Valid EV Obs Band 1 (%)",
		  DFNT_FLOAT32, 1, &ValidEVobservations[0] ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for 'Valid EV Obs Band 1 (%)'");

    if( SDsetattr(sds_id, "Valid EV Obs Band 2 (%)",
		  DFNT_FLOAT32, 1, &ValidEVobservations[1] ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for 'Valid EV Obs Band 2 (%)'");

    if( SDsetattr(sds_id, "Valid EV Obs Band 4 (%)",
		  DFNT_FLOAT32, 1, &ValidEVobservations[3] ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for 'Valid EV Obs Band 4 (%)'");

    if( SDsetattr(sds_id, "Valid EV Obs Band 6 (%)",
		  DFNT_FLOAT32, 1, &ValidEVobservations[5] ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for 'Valid EV Obs Band 6 (%)'");


    if( SDsetattr(sds_id, "Saturated EV Obs Band 1 (%)",
		  DFNT_FLOAT32, 1, &SaturatedEVobservations[0] ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for 'Saturated EV Obs Band 1 (%)'");

    if( SDsetattr(sds_id, "Saturated EV Obs Band 2 (%)",
		  DFNT_FLOAT32, 1, &SaturatedEVobservations[1] ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for 'Saturated EV Obs Band 2 (%)'");

    if( SDsetattr(sds_id, "Saturated EV Obs Band 4 (%)",
		  DFNT_FLOAT32, 1, &SaturatedEVobservations[3] ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for 'Saturated EV Obs Band 4 (%)'");

    if( SDsetattr(sds_id, "Saturated EV Obs Band 6 (%)",
		  DFNT_FLOAT32, 1, &SaturatedEVobservations[5] ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for 'Saturated EV Obs Band 6 (%)'");


    {
      /*** Set AutomaticQualityFlagExplanation. ***/
      char	BadStg[255] = "Exceeded limits:";
      char	AddStg[128];

      if( AutomaticQualityFlag != PASSED ) {

	if( cnt.InvalidInput * 100. / cnt.AllPixelsProcessed + 0.5 > MAX_ACCEPTABLE ) {
	  AutomaticQualityFlag = SUSPECT;
	  strcpy( AddStg, " Invalid_input_data(%)");
	  if( strlen(BadStg) + strlen(AddStg) < 255 )
	    strcat( BadStg, AddStg );
	}

	if( cnt.NoDecision * 100. / cnt.AllPixelsProcessed + 0.5 > MAX_ACCEPTABLE ) {
	  AutomaticQualityFlag = SUSPECT;
	  strcpy( AddStg, " No_decision(%)");
	  if( strlen(BadStg) + strlen(AddStg) < 255 )
	    strcat( BadStg, AddStg );
	}

	strcpy( km.AutomaticQualityFlagExplanation, BadStg);
	PGS_SMF_GenerateStatusReport(
	   "Data quality range limits exceeded. Data should be investigated.");
      }
    }

    if( SDendaccess( sds_id) == FAIL )
      HALT2(ACT09,"SDendaccess failure for Snow_Cover");


    /*** Do SnowQA ***/

    /*** Write out the "Snow_Cover_Pixel_QA" SDS attributes. ***/
    if( (sds_index = SDnametoindex( SnowHDFhandle, "Snow_Cover_Pixel_QA" )) == FAIL )
      HALT2(ACT09,"Cannot find SD index for Snow_Cover_Pixel_QA in the output file");

    if( (sds_id = SDselect( SnowHDFhandle, sds_index)) == FAIL )
      HALT2(ACT09,"SDselect failure for Snow_Cover_Pixel_QA");

    if(SDsetdatastrs(sds_id, longnameQA, units, format, coordsys) == FAIL)
      HALT2(ACT09,"SDsetdatastrs failed for Snow_Cover_Pixel_QA");

    if( SDsetattr(sds_id,"valid_range", DFNT_UINT8, 2, RangeValsQA ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for valid_range for Snow_Cover_Pixel_QA");

    if( SDsetattr(sds_id, "_FillValue", DFNT_UINT8, 1, &FillValue ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for _FillValue for Snow_Cover_Pixel_QA");

    {
      char *s="0=good quality, 1=other quality, 252=Antarctica mask, 253=land mask, 254=ocean mask, 255=fill";
      if( SDsetattr(sds_id, "Key:", DFNT_CHAR8, strlen(s), s ) == FAIL )
	HALT2(ACT09,"SDsetattr failure for Key for Snow_Cover");
    }

    /*** cnt.QA_Other is kept uptodate by UpdateCounts.c. ***/
    if( cnt.LandPixels > 0 ) {
      pct_nominal  = ((float32)(cnt.LandPixels-cnt.QA_Other) /
		      (float32)cnt.LandPixels) * 100.0;
      pct_abnormal = 100 - pct_nominal;
    } else {
      pct_nominal = 0;		/*** All ocean granule ***/
      pct_abnormal = 100;
    }


    /*** Adjust either nominal or invalid so that the sum of the 4 equals 100%. ***/

    if( SDendaccess( sds_id) == FAIL )
      HALT2(ACT09,"SDendaccess failure for SnowQA");


    /*** Write out the "Fractional_Snow_Cover" SDS attributes. ***/

    if( (sds_index = SDnametoindex( SnowHDFhandle,"Fractional_Snow_Cover" )) == FAIL )
      HALT2(ACT09,"Can't find SD index for Fractional_Snow_Cover in the output file");

    if( (sds_id = SDselect( SnowHDFhandle, sds_index)) == FAIL )
      HALT2(ACT09,"SDselect failure for Fractional_Snow_Cover");

    if(SDsetdatastrs(sds_id, longnameFrac, units, format, coordsys) == FAIL)
      HALT2(ACT09,"SDsetdatastrs failed for Fractional_Snow_Cover");

    if( SDsetattr(sds_id,"valid_range", DFNT_UINT8, 2, RangeVals ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for valid_range for Fractional_Snow_Cover");

    if( SDsetattr(sds_id, "_FillValue", DFNT_UINT8, 1, &FillValue ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for _FillValue for Fractional_Snow_Cover");

    sprintf(KeyStg,"0-100=fractional snow, 200=missing data, 201=no decision, 211=night, 225=land, 237=inland water, 239=ocean, 250=cloud, 254=detector saturated, 255=fill");

    if( SDsetattr(sds_id, "Key:", DFNT_CHAR8, strlen(KeyStg), KeyStg ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for Key for Fractional_Snow_Cover");

    if( SDsetattr(sds_id, "Nadir_data_resolution", DFNT_CHAR8,
		  strlen(nadirName), nadirName ) == FAIL )
      HALT2(ACT09,"SDsetattr failure for Nadir_data_resolution for Fractional_Snow_Cover");

    if( SDendaccess( sds_id) == FAIL )
      HALT2(ACT09,"SDendaccess failure for Fractional_Snow_Cover");

  }
  /*----------------------------------------------------------------------------------*/

  if( CallType == TERMINATE ) {

    if( SWclose( SWfidout ) == FAIL )
      HALT2(ACT10,"SWclose failed for output file");
  }

}
